package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.CostCenter;
import com.bocloud.cos.intf.CostCenterService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 成本中心
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/12/30 14:41
 */

@RestController
@RequestMapping("/v1/cost/center")
@Tag(name = "成本中心")
public class CostCenterController {


    @Autowired
    private CostCenterService costCenterService;

    /**
     * 获取成本中心列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(summary = "获取成本中心列表")
    public GeneralResult<GridBean<CostCenter>> list(Pager pager) {
        return costCenterService.list(pager);
    }

    /**
     * 查看成本中心详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(summary = "查看成本中心详情")
    public GeneralResult<CostCenter> detail(@PathVariable(value = Common.ID) Long id) {
        return costCenterService.detail(id);
    }

    /**
     * 添加成本中心
     *
     * @param context
     * @return
     */
    @PostMapping
    @Operation(summary = "添加成本中心")
    public Result create(@RequestBody CostCenter costCenter,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return costCenterService.create(costCenter, context);
    }

    /**
     * 修改成本中心
     *
     * @param id
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改成本中心")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody CostCenter costCenter, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return costCenterService.modify(id, costCenter, context);
    }

    /**
     * 删除成本中心
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除成本中心")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return costCenterService.remove(id, context);
    }


}
