package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.Document;
import com.bocloud.cos.intf.DocumentService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.util.List;

/**
 * 使用文档控制器
 *
 * @author tyl
 * @update fanzx
 * @since 2021/3/8 16:26
 */
@RestController
@RequestMapping("/v1/documents")
@Tag(name = "使用文档")
public class DocumentController {
    private final DocumentService documentService;

    public DocumentController(DocumentService documentService) {
        this.documentService = documentService;
    }

    @Operation(summary = "创建使用文档")
    @PostMapping
    public GeneralResult create(@RequestBody Document document, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.create(document, context);
    }

    @Operation(summary = "获取使用文档列表树")
    @GetMapping
    public GeneralResult list(Pager pager,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.list(pager.getParams(), context);
    }

    @Operation(summary = "模糊搜索查询文档树")
    @GetMapping("/search")
    public GeneralResult search(@RequestParam(value = Common.NAME) String name,
                                @RequestParam(value = "category") String category,
                                @RequestParam(value = "status") String status,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.search(name, category, status, context);
    }

    @Operation(summary = "修改文档信息")
    @PutMapping("/{id}")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody Document document,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.modify(id, document, context);
    }

    @Operation(summary = "删除使用文档")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.remove(id, context);
    }

    @Operation(summary = "获取使用文档详情")
    @GetMapping("/{id}")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return documentService.detail(id);
    }

    @PostMapping("/pictures")
    @Operation(summary = "删除使用文档图片")
    public GeneralResult upload(HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new GeneralResult(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        multipartFile.getContentType();
        // 判断文件名
        if (originalFilename == null) {
            return new GeneralResult(false, "文件名不能为空");
        }
        if (!(originalFilename.endsWith(".png") || originalFilename.endsWith(".PNG") || originalFilename.endsWith(".jpg") || originalFilename.endsWith(".JPG"))) {
            return new GeneralResult(false, "文件格式错误，只支持PNG或JPG格式的图片");
        }
        return documentService.upload(multipartFile);
    }

    @DeleteMapping("/pictures")
    @Operation(summary = "删除使用文档图片")
    public GeneralResult upload(@RequestBody List<String> paths) {
        return documentService.delete(paths);
    }

    /**
     * 管理端控制自服务展示
     *
     * @param document
     * @param context
     * @return
     */
    @Operation(summary = "控制自服务展示文档")
    @PatchMapping
    public GeneralResult serviceDisplay(@RequestBody Document document, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.serviceDisplay(document.getDisplay(), context);
    }

    /**
     * 管理端控制自服务展示
     *
     * @param context
     * @return
     */
    @Operation(summary = "控制自服务展示文档")
    @GetMapping("/display")
    public GeneralResult getDisplay(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return documentService.getDisplay(context);
    }
}
