package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.MetaQuota;
import com.bocloud.cos.service.MetaQuotaService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Result;
import com.megatron.common.utils.RequestContexts;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

@RestController
@RequiredArgsConstructor
@Tag(name = "配额项管理")
@RequestMapping("/v1/quota-metas")
public class MetaQuotaController {

    private final MetaQuotaService metaQuotaService;

    @GetMapping
    @Operation(summary = "配额项列表")
    public GeneralResult<GridBean<MetaQuota>> page(Pager pager) {
        GridBean<MetaQuota> gridBean = metaQuotaService.list(pager);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    @PostMapping
    @Operation(summary = "新增配额项")
    public Result create(@RequestBody MetaQuota metaQuota) {
        metaQuotaService.create(metaQuota, RequestContexts.getContext());
        return new Result(true, "创建成功");
    }

    @PutMapping("/{id}")
    @Operation(summary = "修改配额项")
    public Result update(@PathVariable String id, @RequestBody MetaQuota metaQuota) {
        metaQuotaService.modify(metaQuota, RequestContexts.getContext());
        return new Result(true, "修改成功");
    }

    @DeleteMapping("/{id}")
    @Operation(summary = "删除配额项")
    public Result delete(@PathVariable Long id) {
        metaQuotaService.remove(id, RequestContexts.getContext());
        return new Result(true, "删除成功");
    }

    @GetMapping("/{id}")
    @Operation(summary = "配额项详情")
    public GeneralResult<MetaQuota> get(@PathVariable Long id) {
        return metaQuotaService.detail(id);
    }
}
