package com.bocloud.cos.booter.controller;

import com.bocloud.cos.intf.ResourceScreenService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 云资源大屏控制层
 *
 * @author tyl
 * @since 2020/11/30 14:14
 */
@RestController
@RequestMapping("/v1/cloud/resource")
@Tag(name = "资源大屏")
public class ResourceScreenController {
    private final ResourceScreenService resourceScreenService;

    public ResourceScreenController(ResourceScreenService resourceScreenService) {
        this.resourceScreenService = resourceScreenService;
    }

    @GetMapping("/overview")
    @Operation(summary = "资源总体情况")
    public GeneralResult totalOverview(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.totalOverview(context);
    }

    @GetMapping("/used")
    @Operation(summary = "资源使用情况")
    public GeneralResult usedOverview(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.usedOverview(context);
    }

    @GetMapping
    @Operation(summary = "平台情况")
    public GeneralResult platformOverview(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.platformOverview(context);
    }

    @GetMapping("/types")
    @Operation(summary = "获取所有平台类型")
    public GeneralResult vendorTypes(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.vendorTypes(context);
    }

    @GetMapping("/count")
    @Operation(summary = "云资源统计")
    public GeneralResult count(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.count(type, context);
    }

    @GetMapping("/status")
    @Operation(summary = "云主机状态统计")
    public GeneralResult status(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.status(type, context);
    }

    @GetMapping("/apply")
    @Operation(summary = "云主机申请趋势")
    public GeneralResult resourceUsed(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.resourceApply(type, context);

    }

    @GetMapping("/trend")
    @Operation(summary = "云资源使用趋势")
    public GeneralResult pubCloudResourceTrend(@RequestParam(value = "type", required = false) String type, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceScreenService.cloudResourceTrend(type, context);
    }
}
