package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.model.ServiceCatalogTreeVO;
import com.bocloud.cos.service.ServiceCatalogService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 服务目录
 *
 * @author lym
 * @since 2022年5月30日
 */
@RequiredArgsConstructor
@RestController
@RequestMapping("/v1/services/catalogs")
@Tag(name = "服务目录管理")
public class ServiceCatalogController {

    private final ServiceCatalogService serviceCatalogService;

    /**
     * 添加服务目录
     *
     * @param catalog 服务目录
     * @param context 当前用户
     * @return 操作结果
     */
    @PostMapping
    @Operation(summary = "添加服务目录")
    public Result create(@RequestBody ServiceCatalog catalog, @Value(Common.REQ_CONTEXT) RequestContext context) {
        catalog.setCreatorId(context.getTarget());
        catalog.setMenderId(context.getTarget());
        return serviceCatalogService.create(catalog);
    }

    /**
     * 编辑服务目录
     *
     * @param id      ID
     * @param catalog 服务目录
     * @param context 当前用户
     * @return 操作结果
     */
    @PutMapping("/{id}")
    @Operation(summary = "编辑服务目录")
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody ServiceCatalog catalog,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        catalog.setId(id);
        catalog.setMenderId(context.getTarget());
        return serviceCatalogService.modify(catalog);
    }

    /**
     * 移除服务目录
     *
     * @param id      ID
     * @param context 当前用户
     * @return 操作结果
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除服务目录")
    public Result remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCatalogService.remove(id, context);
    }

    /**
     * 查询服务目录详情
     *
     * @param id ID
     * @return 详情
     */
    @GetMapping("/{id}")
    @Operation(summary = "获取服务目录详情")
    public GeneralResult<ServiceCatalog> detail(@PathVariable Long id) {
        ServiceCatalog serviceCatalog = serviceCatalogService.detail(id);
        return new GeneralResult<>(true, serviceCatalog, "查询成功");
    }

    /**
     * 获取服务目录树
     *
     * @return 服务目录树
     */
    @GetMapping
    @Operation(summary = "获取服务目录树")
    public GeneralResult<List<ServiceCatalogTreeVO>> tree() {
        return serviceCatalogService.tree();
    }
}
