package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.intf.ServiceCategoryService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 服务类型controller
 *
 * @author Administrator
 */
@RestController
@RequestMapping("/v1/services/categories")
@Tag(name = "服务类型管理")
public class ServiceCategoryController {

    private final ServiceCategoryService serviceCategoryService;

    public ServiceCategoryController(ServiceCategoryService serviceCategoryService) {
        this.serviceCategoryService = serviceCategoryService;
    }

    /**
     * 获取服务类型
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(summary = "获取服务类型列表")
    public GeneralResult<GridBean<ServiceCategory>> list(Pager pager) {
        return serviceCategoryService.list(pager);
    }


    /**
     * 获取服务产品类型树
     *
     * @return
     */
    @GetMapping("/tree")
    @Operation(summary = "获取服务产品类型树")
    public GeneralResult<JSONArray> tree() {
        return serviceCategoryService.tree();
    }


    /**
     * 获取服务类型列表
     *
     * @param pager
     * @return
     */
    @GetMapping("/condition")
    @Operation(summary = "获取服务产品列表")
    public GeneralResult<List<ServiceCategory>> listByCondition(Pager pager) {
        return serviceCategoryService.list(pager.getParams());
    }

    /**
     * 添加服务类型
     *
     * @param serviceCategory
     * @param context
     * @return
     */
    @PostMapping
    @Operation(summary = "添加服务类型")
    public Result create(@RequestBody ServiceCategory serviceCategory,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCategoryService.create(serviceCategory, context);

    }

    /**
     * 服务编辑
     *
     * @param id
     * @param serviceCategory
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @Operation(summary = "编辑服务")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody ServiceCategory serviceCategory,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCategoryService.modify(id, serviceCategory, context);
    }

    /**
     * 移除服务类型
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除服务类型")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceCategoryService.remove(id, context.getTarget());
    }

    /**
     * 查询服务目录详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(summary = "获取服务类型详情")
    public GeneralResult<ServiceCategory> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceCategoryService.detail(id);
    }

}
