package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.SkuSpec;
import com.bocloud.cos.intf.SkuSpecService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/skus/specs")
@Tag(name = "产品规格管理")
public class SkuSpecController {

    private final SkuSpecService skuSpecService;

    public SkuSpecController(SkuSpecService skuSpecService) {
        this.skuSpecService = skuSpecService;
    }

    /**
     * 获取产品规格列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(summary = "获取产品规格列表")
    public GeneralResult<GridBean<SkuSpec>> list(Pager pager) {
        return skuSpecService.list(pager);
    }

    /**
     * 查看产品规格详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @Operation(summary = "查看产品规格详情")
    public GeneralResult<SkuSpec> detail(@PathVariable(value = Common.ID) Long id) {
        return skuSpecService.detail(id);
    }

    /**
     * 查看产品规格列表
     *
     * @param code
     * @param categoryId
     * @return
     */
    @GetMapping("/listbycondition")
    @Operation(summary = "查看产品规格列表")
    public GeneralResult listSpecs(@RequestParam(value = Common.CODE, required = false) String code,
                                   @RequestParam(value = "categoryId", required = false) Long categoryId) {
        return skuSpecService.listSpecs(code, categoryId);
    }

    /**
     * 查看产品规格列表
     *
     * @param code
     * @param serverId
     * @param type
     * @return
     */
    @GetMapping("/listbycode")
    @Operation(summary = "查看产品规格列表")
    public GeneralResult<List<SkuSpec>> listSpecsByCode(@RequestParam(value = Common.CODE, required = false) String code,
                                                        @RequestParam(value = "serverId", required = false) Long serverId,
                                                        @RequestParam(value = Common.TYPE, required = false) String type,
                                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        return skuSpecService.listSpecsByCode(serverId, code, type, context);
    }
}
