package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.AccountRecord;
import com.bocloud.cos.entity.TenantAccount;
import com.bocloud.cos.intf.TenantAccountService;
import com.bocloud.cos.params.AccountActionParams;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 租户授信账户 controller
 *
 * @author tyl
 * @time 2020/7/15 周三 19:51
 */
@RestController
@RequestMapping("/v1/tenants")
@Tag(name = "授信账号管理")
public class TenantAccountController {
    private final TenantAccountService tenantAccountService;

    public TenantAccountController(TenantAccountService tenantAccountService) {
        this.tenantAccountService = tenantAccountService;
    }

    @PostMapping("/account")
    @Operation(summary = "租户授信账号")
    public Result assignAccount(@RequestBody TenantAccount tenantAccount, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.assignAccount(tenantAccount, requestContext);
    }

    @DeleteMapping("/{id}/account")
    @Operation(summary = "删除租户授信账号")
    public Result deleteAccount(@PathVariable(Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.deleteAccount(id, requestContext);
    }

    @PutMapping("/account")
    @Operation(summary = "租户授信账号操作")
    public Result accountAction(@RequestBody AccountActionParams params, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.accountAction(params.getTenantId(), params.getBillId(), params.getAction(), params.getMoney(), requestContext);
    }

    @GetMapping("/{id}/account")
    @Operation(summary = "查询租户授信详情")
    public GeneralResult<TenantAccount> accountDetail(@PathVariable(Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.accountDetail(id, requestContext);
    }

    @GetMapping("/account/records")
    @Operation(summary = "查询授信账号记录")
    public GeneralResult<List<AccountRecord>> accountRecord(@RequestParam(value = "tenantId") Long tenantId,
                                                            @RequestParam(value = "action") String action,
                                                            @RequestParam(value = "startTime") String startTime,
                                                            @RequestParam(value = "endTime") String endTime,
                                                            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantAccountService.accountRecord(tenantId, action, startTime, endTime, requestContext);
    }
}
