package com.bocloud.cos.booter.controller;

import com.bocloud.cos.entity.CloudVendorAccount;
import com.bocloud.cos.intf.VendorAccountService;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 平台账号配置管理
 *
 * @author lym
 * @since 2022-06-16
 */
@RestController
@RequestMapping("/v1/vendor/account")
@Tag(name = "账号配置管理")
@Slf4j
public class VendorAccountController {

    @Autowired
    private VendorAccountService vendorAccountService;

    /**
     * 账号配置列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(summary = "获取账号配置列表")
    public GeneralResult<GridBean<CloudVendorAccount>> list(Pager pager) {
        return vendorAccountService.list(pager);
    }


    /**
     * 新增账号配置
     *
     * @param vendorAccount
     * @param context
     * @return
     */
    @Operation(summary = "新增账号配置")
    @PostMapping
    public Result create(@RequestBody CloudVendorAccount vendorAccount,
                         @Value(Common.REQ_CONTEXT) RequestContext context) throws Exception {
        vendorAccount.setCreatorId(context.getTarget());
        vendorAccount.setMenderId(context.getTarget());
        return vendorAccountService.create(vendorAccount);
    }

    /**
     * 编辑账号配置信息
     *
     * @param id
     * @param vendorAccount
     * @param context
     * @return
     */
    @Operation(summary = "编辑账号配置信息")
    @PutMapping("/{id}")
    public Result modify(@RequestBody CloudVendorAccount vendorAccount,
                         @PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) throws Exception {
        vendorAccount.setId(id);
        vendorAccount.setMenderId(context.getTarget());
        return vendorAccountService.modify(vendorAccount);
    }

    /**
     * 移除账号配置
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "删除账号配置")
    @DeleteMapping("/{id}")
    public Result removeByTenantId(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vendorAccountService.remove(id, context);
    }

    /**
     * 获取账号配置详情
     *
     * @param id
     * @return
     */
    @Operation(summary = "获取账号配置详情")
    @GetMapping("/{id}")
    public GeneralResult<CloudVendorAccount> detail(@PathVariable(value = Common.ID) Long id) {
        return vendorAccountService.detail(id);
    }

    /**
     * 导入账号配置
     *
     * @param accounts
     * @return
     */
    @Operation(summary = "获取账号配置详情")
    @PostMapping("/import")
    public Result importAccount(@RequestBody List<CloudVendorAccount> accounts) {
        return vendorAccountService.importAccount(accounts);
    }

    /**
     * 获取所有平台类型
     *
     * @return
     */
    @Operation(summary = "获取所有平台类型")
    @GetMapping("/types")
    public Result importAccount() {
        return vendorAccountService.queryVendorTypes();
    }


}
