package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.entity.assess.AssessClassify;
import com.bocloud.cos.service.assess.AssessClassifyService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author wpj
 * @since 2023/10/30
 */
@RestController
@RequestMapping("/v1/assess/classify")
@Tag(name = "考核分类管理")
public class AssessClassifyController {

    @Autowired
    AssessClassifyService assessClassifyService;

    @Operation(summary = "获取考核分类列表")
    @GetMapping("/list")
    public GeneralResult<GridBean<AssessClassify>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessClassify> result = assessClassifyService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "新增考核分类")
    @PostMapping
    public GeneralResult<?> save(@RequestBody AssessClassify assessClassify, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessClassifyService.save(assessClassify, context);
        return new GeneralResult<>(true, "新增考核分类成功");
    }

    @Operation(summary = "编辑考核分类")
    @PatchMapping
    public GeneralResult<?> edit(@RequestBody AssessClassify assessClassify, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessClassifyService.update(assessClassify, context);
        return new GeneralResult<>(true, "编辑考核分类成功");
    }

    @Operation(summary = "移除考核分类")
    @DeleteMapping("/{id}")
    public GeneralResult<?> remove(@PathVariable(value = Common.ID) Long id) {
        assessClassifyService.remove(id);
        return new GeneralResult<>(true, "移除考核规则成功");
    }
}
