package com.bocloud.cos.booter.controller.assess;

import com.bocloud.cos.entity.assess.AssessRecord;
import com.bocloud.cos.entity.assess.AssessRecordMetric;
import com.bocloud.cos.service.assess.AssessRecordService;
import com.bocloud.cos.entity.Attachment;
import com.bocloud.cos.service.AttachmentService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;


/**
 * 考核记录访问控制
 *
 * @author lym
 */
@Slf4j
@RestController
@RequestMapping("/v1/assess/record")
@Tag(name = "考核记录管理")
public class AssessRecordController {

    @Autowired
    private AssessRecordService assessRecordService;
    @Autowired
    private AttachmentService attachmentService;

    @Operation(summary = "获取考核记录(子任务)列表")
    @GetMapping("/list")
    public GeneralResult<GridBean<AssessRecord>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        GridBean<AssessRecord> result = assessRecordService.list(pager, context);
        return new GeneralResult<>(true, result, "查询成功");
    }

    @Operation(summary = "获取考核任务记录指标详情")
    @GetMapping("/metric/detail/{id}")
    public GeneralResult<List<AssessRecordMetric>> metricDetail(@PathVariable(value = Common.ID) Long recordId) {
        List<AssessRecordMetric> list = assessRecordService.metricDetail(recordId);
        return new GeneralResult<>(true, list, "查询成功");
    }

    @Operation(summary = "评分")
    @PostMapping("/score")
    public GeneralResult<Void> updateScore(@RequestBody List<AssessRecordMetric> recordMetrics, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessRecordService.updateScore(recordMetrics, context);
        return new GeneralResult<>(true, "考核评分中...");
    }

    @Operation(summary = "上传材料")
    @PostMapping("/upload")
    public GeneralResult<Void> upload(@RequestParam(value = "recordMetricId") Long recordMetricId,
                                      @RequestParam(value = "file") MultipartFile file,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            Attachment attachment = attachmentService.upload(file, context);
            Long attachmentId = null;
            if (null != attachment && attachment.getId() != null) {
                attachmentId = attachment.getId();
            }
            assessRecordService.updateAttachmentId(recordMetricId, attachmentId, context);
            return new GeneralResult<>(true, "上传成功");
        } catch (Exception e) {
            log.error("上传材料失败：{}", e);
            return new GeneralResult<>(true, "上传材料失败");
        }
    }

    @Operation(summary = "删除上传材料")
    @PutMapping("/materials")
    public GeneralResult<Void> upload(@RequestParam(value = "recordMetricId") Long recordMetricId,
                                      @RequestParam(value = "attachmentId") Long attachmentId,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            boolean flag = assessRecordService.updateAttachmentId(recordMetricId, null, context);
            if (flag) {
                attachmentService.delete(attachmentId);
            }
            return new GeneralResult<>(true, "删除材料成功");
        } catch (Exception e) {
            log.error("删除材料失败：{}", e);
            return new GeneralResult<>(true, "删除材料失败");
        }
    }

    @Operation(summary = "接收考核")
    @PutMapping("/receive/{id}")
    public GeneralResult<?> receiveAssess(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessRecordService.receiveAssess(id, context);
        return new GeneralResult<>(true, "接收考核成功");
    }

    @Operation(summary = "提交考核任务")
    @PutMapping("/submit/{id}")
    public GeneralResult<?> submitAssess(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        assessRecordService.submitAssess(id, context);
        return new GeneralResult<>(true, "任务提交成功");
    }

    @GetMapping("/{id}/export")
    @Operation(summary = "考核任务历史导出")
    public GeneralResult<Void> download(@PathVariable Long id, HttpServletResponse response) {
        return assessRecordService.download(id, response);
    }

}
