package com.bocloud.cos.booter.scheduler;

import com.bocloud.cos.entity.Coupon;
import com.bocloud.cos.entity.Promotion;
import com.bocloud.cos.repository.CouponRepository;
import com.bocloud.cos.repository.PromotionRepository;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Component
@Slf4j
public class PromotionScheduler {

    @Autowired
    private PromotionRepository promotionRepository;
    @Autowired
    private CouponRepository couponRepository;
    @Autowired
    private CurrentService currentService;

    @Scheduled(cron = "0 0/10 * * * ?")
    protected void monitor() {
        if (!currentService.isLeader()) {
            return;
        }
        try {
            List<Promotion> promotions = promotionRepository.listAll()
                    .stream()
                    .filter(promotion -> !promotion.getStatus().equals(Promotion.Status.expired.name()))
                    .collect(Collectors.toList());
            for (Promotion promotion : promotions) {
                if (promotion.getStatus().equals(Promotion.Status.not_started.name()) && new Date().after(promotion.getStartDate())) {
                    log.info("优惠活动{}已开始", promotion.getName());
                    // 已结束
                    promotion.setStatus(Promotion.Status.progress.name());
                    promotionRepository.update(promotion);
                }
                if (promotion.getStatus().equals(Promotion.Status.progress.name()) && new Date().after(promotion.getEndDate())) {
                    log.info("优惠活动{}已过期", promotion.getName());
                    // 已结束
                    promotion.setStatus(Promotion.Status.expired.name());
                    promotionRepository.update(promotion);
                }
            }

            List<Coupon> coupons = couponRepository.listAll()
                    .stream()
                    .filter(coupon -> coupon.getStatus().equals(Coupon.Status.unused.name()))
                    .collect(Collectors.toList());
            for (Coupon coupon : coupons) {
                if (new Date().after(coupon.getExpiredTime())) {
                    log.info("优惠券{}已过期", coupon.getName());
                    // 已过期
                    coupon.setStatus(Coupon.Status.expired.name());
                    couponRepository.update(coupon);
                }
            }
        } catch (Exception e) {
            log.error("Get error message：{}", e);
        }
    }

}
