package com.bocloud.cos.entity;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 优惠管理
 *
 */
@NoArgsConstructor
@AllArgsConstructor
@Data
@Table("promotion")
public class Promotion extends Generic {

	/**
	 * id
	 */
	@PK(PKStrategy.AUTO)
	private Long id;
	/**
	 * 活动名称
	 */
	@Column("name")
	private String name;

	/**
	 * 活动状态：未开始，进行中，已过期
	 */
	@Column("status")
	private String status;
	/**
	 * 优惠券折扣，1-10，允许小数
	 */
	@Column("discount")
	private BigDecimal discount;
	/**
	 * 优惠券剩余库存
	 */
	@Column("stock")
	private Long stock;
	/**
	 * 优惠券总量
	 */
	@Column("quota")
	private Long quota;
	/**
	 * 创建时间
	 */
	@IgnoreUpdate
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@Column("gmt_create")
	private Date gmtCreate;
	/**
	 * 活动开始时间
	 */
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@Column("start_date")
	private Date startDate;
	/**
	 * 活动结束时间
	 */
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@Column("end_date")
	private Date endDate;
	/**
	 * 修改时间
	 */
	@JsonSerialize(using = DateSerializer.class)
	@JsonDeserialize(using = DateDeserializer.class)
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@Column("gmt_modify")
	private Date gmtModify;
	/**
	 * 创建人id
	 */
	@Column("creator_id")
	private Long creatorId;
	/**
	 * 是否删除
	 */
	@Column("is_deleted")
	private Boolean deleted;

	public enum Status {
		not_started, // 未开始
		progress, // 进行中
		expired// 已过期
	}
}
