package com.bocloud.cos.entity;

import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 配额历史实体类
 *
 * @author tyl
 * @time 2020/3/30 周一 16:22
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@Table("quota_history")
public class QuotaHistory extends Generic {
    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 配额目标id
     */
    @Column("target")
    private Long target;
    @Column("value")
    private Long value;//配额操作值
    @Column("code")
    private String code;// 配额类型
    @Column("unit")
    private String unit;//单位
    @Column("category")
    private ResourceQuota.Category category;//配额类型
    @Column("action")
    private ResourceQuota.Action action;//操作类型
    @Column("gmt_create")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;
    /**
     * 修改者id
     */
    @Column("mender_id")
    private Long menderId;
    @IgnoreAll
    private String serviceName;
    @IgnoreAll
    private String tenantName;//租户名称

    public QuotaHistory(ResourceQuota resourceQuota, ResourceQuota.Action action, Long menderId) {
        this.target = resourceQuota.getTarget();
        this.value = resourceQuota.getQuotaActionValue();
        this.code = resourceQuota.getCode();
        this.unit = resourceQuota.getUnit();
        this.category = resourceQuota.getCategory();
        this.action = action;
        this.gmtCreate = new Date();
        this.menderId = menderId;
    }

    public QuotaHistory(ResourceQuota resourceQuota, Long value, ResourceQuota.Action action, Long menderId) {
        this.target = resourceQuota.getTarget();
        this.code = resourceQuota.getCode();
        this.unit = resourceQuota.getUnit();
        this.category = resourceQuota.getCategory();
        this.action = action;
        this.gmtCreate = new Date();
        this.value = value;
        this.menderId = menderId;
    }

    public QuotaHistory(Long target, Long value, String code, String unit, ResourceQuota.Category category, ResourceQuota.Action action, Date gmtCreate, Long menderId) {
        this.target = target;
        this.value = value;
        this.code = code;
        this.unit = unit;
        this.category = category;
        this.action = action;
        this.gmtCreate = gmtCreate;
        this.menderId = menderId;
    }
}
