package com.bocloud.cos.entity;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 服务目录
 *
 * @author lym
 * @since 2022年5月30日
 */
@EqualsAndHashCode(callSuper = true)
@Data
@Table("service_catalog")
@NoArgsConstructor
@AllArgsConstructor
public class ServiceCatalog extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id;

    @Column("name")
    private String name;

    @Column("sort")
    private Integer sort;

    @Column("remark")
    private String remark;

    @Column("parent_id")
    private Long parentId;

    @Column("is_deleted")
    private Boolean deleted;

    @Column("creator_id")
    private Long creatorId;

    @Column("mender_id")
    private Long menderId;

    @Column(value = "gmt_create", nullable = false)
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * SN
     */
    @Column("sn")
    private String sn;
    /**
     * 编码
     */
    @Column("code")
    private String code;
    /**
     * 是否有产品
     */
    @Column("has_category")
    private Boolean hasCategory;
    /**
     * 计费项
     */
    @Column("props")
    private String props;
    /**
     * 当前目录下的服务
     */
    @IgnoreAll
    private List<CloudService> cloudServices;

}
