package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 考核规则实体类
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_rule")
public class AssessRule extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 所属指标ID
     */
    @Column("metric_id")
    private Long metricId;
    /**
     * 规则内容
     */
    @Column("rule_content")
    private String ruleContent;
    /**
     * 提交材料项
     */
    @Column("submission_item")
    private String submissionItem;
    /**
     * 数据函数
     */
    @Column("data_function")
    private String dataFunction;
    /**
     * 评分方式（数据区间、提交材料满足项等）
     */
    @Column("scoring_manner")
    private String scoringManner;
    /**
     * 分值设定
     */
    @Column("score_setting")
    private String scoreSetting;
    /**
     * 分值设定
     */
    @Column("monitor_metric_name")
    private String monitorMetricName;
    /**
     * 最大分值
     */
    @Column("max_score")
    private Integer maxScore;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    @Getter
    public enum ScoringMannerEnum {
        // 评分方式
        SUBMIT_MATERIAL_SATISFACTION_ITEMS("提交材料满足项", 1),
        QUANTITY_INTERVAL("数量区间", 2),
        REPORT_SUBMISSION_STATUS("报告提交情况", 3);

        final String name;
        final int value;

        ScoringMannerEnum(String name, int value) {
            this.name = name;
            this.value = value;
        }
    }
}
