package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.*;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 考核任务
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_task")
public class AssessTask extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 考核任务名称
     */
    @Column("name")
    private String name;
    /**
     * 考核分类ID
     */
    @Column("classify_id")
    private Long classifyId;
    /**
     * 考核周期类型（月度、季度、年度）
     */
    @Column("cycle_type")
    private String cycleType;
    /**
     * 考核周期值
     */
    @Column("cycle_value")
    private String cycleValue;
    /**
     * 状态
     */
    @Column("status")
    private String status;
    /**
     * 绩效标准ID
     */
    @Column("conformity_standard_id")
    private Long standardId;
    /**
     * 任务所选的考核对象（仅供前端回显选中使用）
     */
    @Column("object_path")
    private String objectPath;
    /**
     * 模板ID
     */
    @IgnoreAll
    private Long templateId;
    /**
     * 是否是模版创建（true：是；false：否）
     */
    @Column("is_template")
    private Boolean isTemplate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 备注
     */
    @Column("remark")
    private String remark;

    /**
     * 考核分类名称
     */
    @IgnoreAll
    private String className;
    /**
     *  考核绩效标准
     */
    @IgnoreAll
    private String conformityStandard;
    /**
     * 考核对象集
     */
    @IgnoreAll
    List<AssessObject> objectList;
    /**
     * 考核对象数量
     */
    @IgnoreAll
    private Long objNum;
    /**
     * 考核任务指标数量
     */
    @IgnoreAll
    private Long metricNum;
    /**
     * 创建人
     */
    @IgnoreAll
    private String creator;

    @Getter
    public enum CycleTypeEnum {
        // 考核周期类型
        YEAR("年度考核", "YEAR"),
        QUARTER("季度考核", "QUARTER"),
        MONTH("月度考核", "MONTH");

        final String name;
        final String value;

        CycleTypeEnum(String name, String value) {
            this.name = name;
            this.value = value;
        }
    }
}
