package com.bocloud.cos.entity.assess;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 考核任务与考核指标信息映射类
 *
 * @author lym
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("assess_task_metric")
public class AssessTaskMetric extends Generic {
    /**
     * id
     */
    @PK(PKStrategy.AUTO)
    private Long id;
    /**
     * 考核ID
     */
    @Column("task_id")
    private Long taskId;
    /**
    /**
     * 考核指标ID
     */
    @Column("metric_id")
    private Long metricId;
    /**
     * 指标细则ID
     */
    @Column("metric_rule_id")
    private Long metricRuleId;
    /**
     * 权重
     */
    @Column("weight")
    private Integer weight;
    /**
     * 最大分值
     */
    @Column("max_score")
    private Integer maxScore;
    /**
     * 最终评分值所属的评分等级（用于前端回显选中）
     */
    @Column("score_level")
    private Integer scoreLevel;
    /**
     * 创建时间
     */
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_create")
    private Date gmtCreate;
    /**
     * 创建者ID
     */
    @Column("creator_id")
    private Long creatorId;
    /**
     * 修改者ID
     */
    @Column("mender_id")
    private Long menderId;
    /**
     * 修改时间
     */
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Column("gmt_modify")
    private Date gmtModify;
    /**
     * 备注
     */
    @Column("remark")
    private String remark;

    /**
     * 指标名称
     */
    @IgnoreAll
    private String metricName;
    /**
     * 考核规则内容
     */
    @IgnoreAll
    private String ruleContent;
    /**
     * 提交材料项
     */
    @IgnoreAll
    private String submissionItem;
    /**
     * 评分方式（数据区间、提交材料满足项等）
     */
    private String scoringManner;
    /**
     * 分值设定
     */
    @IgnoreAll
    private String scoreSetting;
    /**
     * 数据函数
     */
    private String dataFunction;
    /**
     * 采集方式（AUTO: 自动；MANUAL：手动）
     */
    private String acquisitionMode;

}
