package com.bocloud.cos.repository;

import com.bocloud.cos.entity.Coupon;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;

@Repository("couponRepository")
public class CouponRepository extends BasicGenericDao<Coupon, Long> {


	public CouponRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, npJdbcTemplate, service);
	}

	public List<Coupon> list(int page, int rows, Map<String, String> sorter, List<Param> params)
			throws Exception {
		String sql = "select a.*,p.discount from coupon a left join promotion p on p.id = a.promotion_id where 1=1 ";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(Coupon.class, sql, paramMap);
	}

	public int count(List<Param> params) throws Exception {
		String sql = "select count(a.id) from coupon a  where 1 = 1";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		sql = getQueryBuilder().build(sql, params, null, "a");
		return this.countQuery(sql, paramMap).intValue();
	}

	public Boolean remove(Long id) throws Exception {
		String sql = "delete from coupon where id = :id";
		Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
		return this.execute(sql, params) > 0;
	}

	public List<Coupon> listByPromotionId(Long promotionId) {
		String sql = "select a.* from coupon a  where a.promotion_id = :promotionId";
		Map<String, Object> params = MapTools.simpleMap("promotionId", promotionId);
		return this.list(Coupon.class, sql, params);
	}


	public Boolean removeByPromotionId(Long promotionId) {
		String sql = "delete from coupon where promotion_id = :promotionId";
		Map<String, Object> params = MapTools.simpleMap("promotionId", promotionId);
		return this.execute(sql, params) > 0;
	}

	// 更新优惠券状态
	public boolean updateStatus(Long id, String status, String orderSn) {
		String sql = "update coupon set status = :status, order_sn = :orderSn where id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("status", status);
		params.put("orderSn", orderSn);
		return this.execute(sql, params) > 0;
	}

	// 更新优惠券状态
	public void updateStatus(String status, String oldOrderSn, String newOrderSn) {
		String sql = "update coupon set status = :status, order_sn = :newOrderSn where order_sn = :oldOrderSn";
		Map<String, Object> params = MapTools.simpleMap("status", status);
		params.put("newOrderSn", newOrderSn);
		params.put("oldOrderSn", oldOrderSn);
		this.execute(sql, params);
	}

	public List<Coupon> listAll() {
		String sql = "select a.* from coupon a  where 1 = 1";
		return this.list(Coupon.class, sql, null);
	}

	public List<Coupon> listByPromotionId(Long promotionId, Long tenantId) {
		String sql = "select a.* from coupon a  where a.promotion_id = :promotionId and a.tenant_id = :tenantId";
		Map<String, Object> params = MapTools.simpleMap("promotionId", promotionId);
		params.put("tenantId", tenantId);
		return this.list(Coupon.class, sql, params);
	}

	public Coupon getCouponById(Long id) {
		if (null == id) {
			return null;
		}
		String sql = "select a.*, p.discount from coupon a left join promotion p on a.promotion_id = p.id where a.id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		List<Coupon> list = this.list(Coupon.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public Coupon getCouponByOrderSn(String orderSn) {
		if (!StringUtils.hasText(orderSn)) {
			return null;
		}
		String sql = "select a.*, p.discount from coupon a left join promotion p on a.promotion_id = p.id where a.order_sn = :orderSn";
		Map<String, Object> params = MapTools.simpleMap("orderSn", orderSn);
		List<Coupon> list = this.list(Coupon.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}
}
