package com.bocloud.cos.repository;

import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.bocloud.cos.entity.InteractiveRecord;
import com.megatron.database.core.intf.impl.BasicGenericDao;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 问题沟通记录DAO实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月8日
 */
@Repository
public class InteractiveRecordRepository extends BasicGenericDao<InteractiveRecord, Long> {

    public InteractiveRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<InteractiveRecord> list(int page, int rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select a.* from interactive_record a where 1 = 1 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.list(InteractiveRecord.class, sql, param);
    }

    public List<InteractiveRecord> listByOrderId(Long orderId) {
        String sql = "select a.* from interactive_record a where a.order_id = :orderId order by a.gmt_create asc";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        return this.list(InteractiveRecord.class, sql, params);
    }

    public boolean removeByOrderId(Long orderId) {
        String sql = "delete from interactive_record where order_id = :orderId";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from interactive_record where 1 = 1";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public InteractiveRecord query(Long id) {
        String sql = "select a.* from interactive_record a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<InteractiveRecord> records = this.list(InteractiveRecord.class, sql, params);
        if (ListTool.isEmpty(records)) {
            return null;
        }
        return records.get(0);
    }

}
