package com.bocloud.cos.repository;

import com.bocloud.cos.entity.Promotion;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;


@Repository("promotionRepository")
public class PromotionRepository extends BasicGenericDao<Promotion, Long> {

    public PromotionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Promotion> list(int page, int rows, Map<String, String> sorter, List<Param> params) throws Exception {
        String sql = "select a.* from promotion a where a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(Promotion.class, sql, paramMap);
    }


    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from promotion a where a.is_deleted = 0  ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<Promotion> listTenant(int page, int rows, Map<String, String> sorter, List<Param> params, Long tenantId) throws Exception {
        String sql = "select a.* from promotion a where a.is_deleted = 0 and a.id in (select promotion_id from coupon where is_deleted = 0 and tenant_id = " + tenantId + ") ";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(Promotion.class, sql, paramMap);
    }


    public int countTenant(List<Param> params, Long tenantId) throws Exception {
        String sql = "select count(1) from promotion a where a.is_deleted = 0 and a.id in (select promotion_id from coupon where is_deleted = 0 and tenant_id = " + tenantId + ")   ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }


    public Boolean remove(Long id) throws Exception {
        String sql = "update promotion set is_deleted = 1, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public Promotion query(Long id) throws Exception {
        String sql = "select a.* from promotion a  where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Promotion> list = this.list(Promotion.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean existName(String name) {
        String sql = "select * from promotion where name = :name and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<Promotion> list = this.list(Promotion.class, sql, params);
        if (list.isEmpty()) {
            return false;
        }
        return true;
    }

    public boolean updateStatus(Long id, String status) throws Exception {
        String sql = "update promotion set status = :status, gmt_modify = :gmtModify where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public List<Promotion> listAll() {
        String sql = "select a.* from promotion a  where a.is_deleted = 0 ";
        return this.list(Promotion.class, sql, null);
    }
}
