package com.bocloud.cos.repository;

import cn.hutool.log.Log;
import com.bocloud.cos.entity.ServiceAddress;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 服务商
 *
 * @author hqy
 * @version 1.0
 * @since 2023年11月15日
 */
@Slf4j
@Repository
public class ServiceAddressRepository extends JdbcGenericDao<ServiceAddress, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    @Autowired
    public ServiceAddressRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 分页查询通讯录列表
     *
     * @param pager 分页
     * @return 分页结果
     */
    public List<ServiceAddress> list(Pager pager) {
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        String sql = " select a.* from service_address a where a.is_deleted = 0 ";
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ServiceAddress.class, sql, paramMap);
    }


    public List<ServiceAddress> list(Pager pager,List<Long> departIdList) {
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        String sql = " select a.* from service_address a where a.is_deleted = 0 ";
        if(!CollectionUtils.isEmpty(departIdList)){
            sql += " and a.depart_id in (:departIdList) ";
            paramMap.put("departIdList",departIdList);
        }
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return this.list(ServiceAddress.class, sql, paramMap);
    }



    public List<SimpleBean> simplelist(List<Param> params, Map<String, String> sorter, List<Long> departIdList) {

        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        String sql = " select a.* from service_address a where a.is_deleted = 0 ";
        if(!CollectionUtils.isEmpty(departIdList)){
            sql += " and a.depart_id in (:departIdList) ";
            paramMap.put("departIdList",departIdList);
        }
        sql = getQueryBuilder().build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        List<ServiceAddress> list = this.list(ServiceAddress.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (ServiceAddress obj : list) {
            beans.add(new SimpleBean(obj.getId(), obj.getName()));
        }
        return beans;
    }

    /**
     * 查询通讯录列表
     *
     * @param params 查询参数
     * @return 列表结果
     */
    public List<ServiceAddress> list(List<Param> params, Map<String, String> sorter, List<Long> departIdList) {
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        String sql = " select a.* from service_address a where a.is_deleted = 0 ";
        if(!CollectionUtils.isEmpty(departIdList)){
            sql += " and a.depart_id in (:departIdList) ";
            paramMap.put("departIdList",departIdList);
        }
        sql = getQueryBuilder().build(sql, params, sorter, "a");
        return this.list(ServiceAddress.class, sql, paramMap);
    }

    /**
     * 查询通讯录数量
     *
     * @param params 查询参数
     * @return int 数量
     */
    public int count(List<Param> params, List<Long> departIdList) {
        String sql = " select count(1) from service_address a where a.is_deleted = 0 ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        if (!CollectionUtils.isEmpty(departIdList)) {
            sql += " and a.depart_id in (:departIdList) ";
            paramMap.put("departIdList", departIdList);
        }
        sql = getQueryBuilder().build(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 根据查询通讯录详情
     *
     * @param id id
     * @return 详情
     */
    public ServiceAddress query(Long id) {
        String sql = "select a.* from service_address a  where a.id = :id and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ServiceAddress> list = this.list(ServiceAddress.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }


    /**
     * 判断通讯录名称是否存在
     *
     * @param name 名称
     * @return 是否存在
     */
    public boolean existsName(String name, Long excludeId) {
        String sql = "select a.* from service_address a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        if (null != excludeId) {
            sql += " and id != :excludeId ";
            params.put("excludeId", excludeId);
        }
        List<ServiceAddress> list = this.list(ServiceAddress.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断通讯录编码是否存在
     *
     * @param code 编码
     * @return 是否存在
     */
    public boolean existsCode(String code, Long excludeId) {
        String sql = "select a.* from service_address a where a.is_deleted = 0 and a.code = :code ";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        if (null != excludeId) {
            sql += " and id != :excludeId ";
            params.put("excludeId", excludeId);
        }
        List<ServiceAddress> list = this.list(ServiceAddress.class, sql, params);
        return !list.isEmpty();
    }


}
