package com.bocloud.cos.repository;

import com.bocloud.cos.entity.ServiceAddress;
import com.bocloud.cos.entity.ServiceAddressVendor;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 服务商平台
 *
 * @author hqy
 * @version 1.0
 * @since 2023年11月15日
 */
@Slf4j
@Repository
public class ServiceAddressVendorRepository extends BasicGenericDao<ServiceAddressVendor, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    @Autowired
    public ServiceAddressVendorRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据通讯录地址查询配置的的平台
     * @param addressId 通讯录地址
     * @return 平台id列表
     */
    public List<Long> listVendorIdByAddressId(Long addressId){
        String sql = "  select vendor_id from service_address_vendor where address_id = :addressId ";
        Map<String,Object> params = MapTools.simpleMap("addressId",addressId);
        List<ServiceAddressVendor> list = this.list(ServiceAddressVendor.class,sql,params);
        return list.stream().map(ServiceAddressVendor::getVendorId).collect(Collectors.toList());
    }

    public List<Long> listVendorIdByName(String name){
        String sql = "select vendor_id from service_address_vendor where address_id = (select id from service_address where name = '" + name + "')";
        Map<String,Object> params = MapTools.simpleMap("name",name);
        List<ServiceAddressVendor> list = this.list(ServiceAddressVendor.class,sql,params);
        return list.stream().map(ServiceAddressVendor::getVendorId).collect(Collectors.toList());
    }

    /**
     * 根据通讯录地址删除配置的平台关系
     * @param addressId 通讯录地址
     */
    public void removeByAddressId(Long addressId){
        String sql = " delete from service_address_vendor where address_id = :addressId  ";
        Map<String,Object> params = MapTools.simpleMap("addressId",addressId);
        this.execute(sql,params);
    }

    /**
     * 查询列表中已配置的平台
     * @param vendorIds 平台列表
     * @return 已配置的平台列表
     */
    public List<Long> listConfigVendorIds(List<Long> vendorIds){
        String sql = "  select vendor_id from service_address_vendor where vendor_id in (:vendorIds) ";
        Map<String,Object> params = MapTools.simpleMap("vendorIds",vendorIds);
        List<ServiceAddressVendor> list = this.list(ServiceAddressVendor.class,sql,params);
        return list.stream().map(ServiceAddressVendor::getVendorId).collect(Collectors.toList());
    }

    /**
     * 查询所有已配置的平台
     * @return 已配置的平台列表
     */
    public List<Long> listConfigVendorIds(Long excludeId){
        String sql = "  select vendor_id from service_address_vendor ";
        Map<String,Object> params = new HashMap<>();
        if(null != excludeId){
            sql += " where address_id != :excludeId ";
            params.put("excludeId",excludeId);
        }
        List<ServiceAddressVendor> list = this.list(ServiceAddressVendor.class,sql,params);
        return list.stream().map(ServiceAddressVendor::getVendorId).collect(Collectors.toList());
    }

    /**
     * 根据通讯录地址删除配置的平台关系
     * @param addressId 通讯录地址
     * @param vendorIds 平台id列表
     */
    public void removeByVendorIds(List<Long> vendorIds,Long addressId){
        String sql = " delete from service_address_vendor where vendor_id in (:vendorIds) and address_id =:addressId  ";
        Map<String,Object> params = MapTools.simpleMap("addressId",addressId);
        params.put("vendorIds",vendorIds);
        this.execute(sql,params);
    }
}
