package com.bocloud.cos.repository;

import com.bocloud.cos.entity.ServiceCatalog;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * 服务类别数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class ServiceCatalogRepository extends BasicGenericDao<ServiceCatalog, Long> {

    public ServiceCatalogRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<ServiceCatalog> queryChildren(Long parentId) throws DataBaseException {
        String sql = String.format("select * from service_catalog where is_deleted = 0 and parent_id = '%s'", parentId);
        return this.list(ServiceCatalog.class, sql);
    }

    public List<ServiceCatalog> listAll() throws DataBaseException {
        String sql = "select a.* from service_catalog a where a.is_deleted = 0";
        return this.list(ServiceCatalog.class, sql);
    }

    public ServiceCatalog getByCode(String code) throws DataBaseException {
        String sql = "select a.* from service_catalog a where a.is_deleted = 0 and a.code = :code limit 1";
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put("code", code);
        List<ServiceCatalog> catalogs = this.list(ServiceCatalog.class, sql, paramMap);
        return Optional.ofNullable(catalogs).orElse(new ArrayList<>()).stream().findFirst().orElse(null);
    }

    public Integer getMaxId() {
        String sql = "select max(a.id) from service_catalog a";
        Long count = this.countQuery(sql, null);
        return count == null ? 0 : count.intValue();
    }


    /**
     * 判断名称是否存在
     *
     * @param name 名称
     * @return 是否存在
     */
    public boolean existsName(String name) {
        String sql = "select * from service_catalog a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<ServiceCatalog> list = this.list(ServiceCatalog.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断编码是否存在
     *
     * @param code 编码
     * @return 是否存在
     */
    public boolean existsCode(String code) {
        String sql = "select * from service_catalog a where a.is_deleted = 0 and a.code = :code";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<ServiceCatalog> list = this.list(ServiceCatalog.class, sql, params);
        return !list.isEmpty();
    }


}
