package com.bocloud.cos.repository;

import com.bocloud.cos.entity.ServiceSku;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;

import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 商品 sku数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class ServiceSkuRepository extends JdbcGenericDao<ServiceSku, Long> {

    public ServiceSkuRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<ServiceSku> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws DataBaseException {
        String sql = "select a.*,b.name service_name,c.name category_name,d.id catalog_id,d.name catalog_name from service_sku a"
                + " left join cloud_service b on b.id = a.service_id"
                + " left join service_category c on c.id = a.category_id"
                + " left join service_catalog d on d.id = a.catalog_id where a.is_deleted = 0";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ServiceSku.class, sql, paramMap);
    }

    public boolean remove(Long id, Long managerId) throws DataBaseException {
        String sql = "update service_sku set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :managerId where is_deleted = 0 and id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("managerId", managerId);
        params.put("gmtModify", new Date());
        return this.execute(sql, params) > 0;
    }

    public List<ServiceSku> list(List<Param> params) throws DataBaseException {
        String sql = "select * from service_sku a where a.is_deleted = 0";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.list(ServiceSku.class, sql, paramMap);
    }

    public int count(List<Param> params) throws DataBaseException {
        String sql = "select count(a.id) from service_sku a where a.is_deleted = 0";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    public ServiceSku query(Long id) throws DataBaseException {
        String sql = "select a.*,b.name service_name,c.name category_name,c.props category_props,d.id catalog_id,d.name catalog_name from service_sku a"
                + " left join cloud_service b on b.id = a.service_id"
                + " left join service_category c on c.id = a.category_id"
                + " left join service_catalog d on d.id = a.catalog_id  where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
        List<ServiceSku> list = this.list(ServiceSku.class, sql, param);
        return list.isEmpty() ? null : list.get(0);
    }

    public ServiceSku getBySpec(String spec, Long categoryId) throws DataBaseException {
        String sql = "select a.* from service_sku a where a.spec = :spec and a.category_id = :categoryId and a.is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap("spec", spec);
        param.put("categoryId", categoryId);
        List<ServiceSku> list = this.list(ServiceSku.class, sql, param);
        return list.isEmpty() ? null : list.get(0);
    }

    public Integer getMaxId() throws DataBaseException {
        String sql = "select max(a.id) from service_sku a";
        Map<String, Object> paramMap = new HashMap<String, Object>();
        Long count = this.countQuery(sql, paramMap);
        if (null == count) {
            return 0;
        }
        return count.intValue();
    }

    public ServiceSku getBySkuCode(String skuCode) throws DataBaseException {
        String sql = "select a.* from service_sku a where a.service_code = :skuCode and a.is_deleted = 0";
        Map<String, Object> param = MapTools.simpleMap("skuCode", skuCode);
        List<ServiceSku> list = this.list(ServiceSku.class, sql, param);
        return list.isEmpty() ? null : list.get(0);
    }

    public ServiceSku getByCategoryId(Long categoryId) throws DataBaseException {
        String sql = "select a.* from service_sku a where a.category_id = :categoryId and a.is_deleted = 0 and a.status = 'enable'";
        Map<String, Object> param = MapTools.simpleMap("categoryId", categoryId);
        List<ServiceSku> list = this.list(ServiceSku.class, sql, param);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<ServiceSku> listByServiceCode(String serviceCode) throws DataBaseException {
        String sql = "select a.*,b.name service_name,c.name category_name from service_sku a"
                + " left join cloud_service b on b.id = a.service_id"
                + " left join service_category c on c.id = a.category_id where a.is_deleted = 0 and a.status = 'enable' and a.service_code = :serviceCode";
        return this.list(ServiceSku.class, sql, MapTools.simpleMap("serviceCode", serviceCode));
    }

    public boolean updateStatus(Long id, String status, Long userId) throws DataBaseException {
        String sql = "update service_sku set status = :status, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        params.put("status", status);
        return this.execute(sql, params) > 0;
    }

    public boolean exists(String column, String value) throws DataBaseException {
        String sql = "select * from service_sku where is_deleted = 0 and " + column + " = :value";
        Map<String, Object> params = MapTools.simpleMap("value", value);
        List<ServiceSku> list = this.list(ServiceSku.class, sql, params);
        return !list.isEmpty();
    }

    public boolean exists(String column, String value, Long categoryId) throws DataBaseException {
        String sql = "select * from service_sku where is_deleted = 0 and " + column
                + " = :value and category_id = :categoryId";
        Map<String, Object> param = MapTools.simpleMap("categoryId", categoryId);
        param.put("value", value);
        List<ServiceSku> list = this.list(ServiceSku.class, sql, param);
        return !list.isEmpty();
    }

}
