package com.bocloud.cos.repository;

import com.bocloud.cos.entity.ServiceTenant;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/3/4 15:42
 */
@Slf4j
@Repository
public class ServiceTenantRepository extends BasicGenericDao<ServiceTenant, Long> {

    public ServiceTenantRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据服务id获取租户授权服务关系集合
     *
     * @param serviceId 服务id
     * @return 租户授权服务关系集合
     */
    public List<ServiceTenant> listTenant(Long serviceId) {
        String sql = "select * from service_tenant where service_id = :serviceId";
        Map<String, Object> params = MapTools.simpleMap("serviceId", serviceId);
        return this.list(ServiceTenant.class, sql, params);
    }

    public List<ServiceTenant> listByTenantId(Long tenantId) {
        String sql = "select a.*,b.code service_code from service_tenant a left join cloud_service b on a.service_id = b.id where a.tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return this.list(ServiceTenant.class, sql, params);
    }

    /**
     * 根据租户和服务ID查询服务授权信息
     *
     * @param tenantId  租户id
     * @param serviceId 服务id
     * @return ServiceTenant
     */
    public ServiceTenant getByTenant(Long tenantId, Long serviceId) {
        String sql = "select * from service_tenant where service_id = :serviceId and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("serviceId", serviceId);
        params.put("tenantId", tenantId);
        List<ServiceTenant> list = this.list(ServiceTenant.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }
}
