package com.bocloud.cos.repository.assess;

import com.bocloud.cos.entity.assess.AssessRecordMetric;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 考核记录与考核指标细则DAO
 *
 * @author lym
 */
@Repository
public class AssessRecordMetricRepository extends BasicGenericDao<AssessRecordMetric, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    public AssessRecordMetricRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<AssessRecordMetric> queryByRecordId(Long recordId) {
        String sql = "select rm.*, m.name as metric_name, m.acquisition_mode, r.data_function, r.scoring_manner, " +
                " r.metric_id, r.score_setting, r.rule_content, r.submission_item, a.path as upload_path, a.name as upload_materials " +
                " from assess_record_metric rm" +
                " left join assess_rule r on r.id = rm.metric_rule_id" +
                " left join assess_metric m on m.id = r.metric_id" +
                " left join attachment a on rm.attachment_id = a.id" +
                " where rm.record_id = " + recordId;
        return list(AssessRecordMetric.class, sql);
    }

    public List<AssessRecordMetric> queryByRecordIds(List<Long> recordIds) {
        String sql = "select rm.*, m.name as metric_name, m.acquisition_mode, r.score_setting " +
                " from assess_record_metric rm" +
                " left join assess_rule r on r.id = rm.metric_rule_id" +
                " left join assess_metric m on m.id = r.metric_id" +
                " where rm.record_id in (:recordIds)";
        Map<String, Object> params = MapTools.simpleMap("recordIds", recordIds);
        return list(AssessRecordMetric.class, sql, params);
    }

    public List<AssessRecordMetric> detailOfTask(Long recordId) {
        String sql = "select am.name as metric_name, ar.rule_content, ar.submission_item, ar.score_setting, ar.score_type, a.*" +
                " from assess_record_metric a" +
                "         left join assess_metric am on am.id = a.metric_id" +
                "         left join assess_rule ar on ar.id = a.metric_rule_id" +
                " where a.record_id = " + recordId;
        return list(AssessRecordMetric.class, sql);
    }

}
