package com.bocloud.cos.repository.assess;

import com.bocloud.cos.entity.assess.AssessRecord;
import com.google.common.base.Joiner;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 考核记录DAO
 *
 * @author lym
 */
@Repository
public class AssessRecordRepository extends BasicGenericDao<AssessRecord, Long> {

    @Value("${spring.datasource.cmp:cmp}")
    private String cmpDb;

    public AssessRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<AssessRecord> list(int page, int rows, Map<String, String> sorter, List<Param> params, boolean isManager, Long userId) {
        String sql = "SELECT a.*, b.name AS class_name, t.name AS task_name, o.name AS assess_obj, u.name AS creator,u2.name as executor, " +
                "acs.name as conformity_standard, t.cycle_type, t.cycle_value " +
                "FROM assess_record a " +
                "LEFT JOIN assess_object o ON a.assess_obj_id = o.id " +
                "LEFT JOIN assess_task t ON a.task_id = t.id " +
                "LEFT JOIN assess_conformity_standard acs ON t.conformity_standard_id = acs.id " +
                "LEFT JOIN " + cmpDb + ".users u ON a.creator_id = u.id " +
                "LEFT JOIN " + cmpDb + ".users u2 ON a.executor_id = u2.id " +
                "LEFT JOIN assess_classify b ON t.classify_id = b.id " +
                "WHERE a.is_deleted = 0 ";
        if (!isManager) {
            sql += generateSubSql(userId);
        }
        sql = getQueryBuilder()
                .buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder()
                .getParam(params);
        return this.list(AssessRecord.class, sql, paramMap);
    }

    public String generateSubSql(Long userId) {
        return " and o.name collate utf8mb4_general_ci in (select t.name" +
                " from (select d.name from cmp.users u" +
                "    left join cmp.department d on u.depart_id = d.id" +
                "    where u.id = " + userId + " and u.is_manager = false" +
                "    union all" +
                "    select s.name from cmp.users u" +
                "    left join service_address s on s.id = u.service_address_id" +
                "    where u.id = " + userId + " and u.is_manager = false" +
                "    union all" +
                "    select p.name from cmp.project p where p.depart_id in (select d.id " +
                "    from cmp.users u " +
                "    left join cmp.department d on u.depart_id = d.id " +
                "    where u.id = " + userId +
                "    and u.is_manager = false)" +
                " ) t)";
    }
    public int count(List<Param> params, boolean isManager, Long userId) {
        String sql = "select count(1) from assess_record a " +
                "left join assess_object o on a.assess_obj_id = o.id " +
                "where a.is_deleted = 0 ";
        if (!isManager) {
            sql += generateSubSql(userId);
        }
        sql = getQueryBuilder().build(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * Retrieves a list of AssessTasks and their scores, ranked in descending order of score.
     *
     * @param cycleValue the cycle type value
     * @return a list of AssessTasks with their scores
     */
    public List<AssessRecord> scoreRanking(Long classifyId, String cycleValue, String sort, Integer limit) {
        String sql = "SELECT o.name as assess_obj , SUM(execute_score) as execute_score " +
                " FROM assess_record a " +
                " LEFT JOIN assess_object o ON a.assess_obj_id = o.id " +
                " LEFT JOIN assess_task t ON a.task_id = t.id " +
                " WHERE a.is_execute = 1 " +
                " AND a.is_deleted = 0 " +
                " AND t.classify_id = " + classifyId +
                " AND t.cycle_value = '" + cycleValue + "' " +
                " GROUP BY assess_obj ORDER BY execute_score ";
        if (!"asc".equals(sort)) {
            sql += " desc";
        }
        if (null != limit) {
            sql += " limit " + limit;
        }
        return list(AssessRecord.class, sql);
    }

    public List<AssessRecord> scoreTrend(Long classifyId, List<String> cycleValues) {
        String sql = "SELECT o.name as assess_obj, t.cycle_value, a.execute_score " +
                " FROM assess_record a " +
                " LEFT JOIN assess_task t ON a.task_id = t.id " +
                " LEFT JOIN assess_object o ON a.assess_obj_id = o.id " +
                " WHERE a.is_deleted = 0 and a.is_execute = 1 and t.classify_id = " + classifyId +
                " AND a.task_id in (select id from assess_task where cycle_value in ('" + Joiner.on("','").join(cycleValues) + "') and classify_id = " + classifyId + ")";
        return list(AssessRecord.class, sql);
    }

    public List<AssessRecord> queryByClassifyIdAndCycleValue(int page, int rows, Map<String, String> sorter, Long classifyId, String cycleValue) {
        String sql = generateSql(classifyId, cycleValue);
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, null, sorter), "t");
        return list(AssessRecord.class, sql, null);
    }

    public List<AssessRecord> queryByClassifyIdAndCycleValue(Long classifyId, String cycleValue) {
        return list(AssessRecord.class, generateSql(classifyId, cycleValue));
    }

    public int countByClassifyIdAndCycleValue(Long classifyId, String cycleValue) {
        String sql = "select count(1) from (" + generateSql(classifyId, cycleValue) + ") as t";
        return countQuery(sql, null).intValue();
    }

    private String generateSql(Long classifyId, String cycleValue) {
        return "SELECT t.*, o.name as assess_obj FROM assess_record t " +
                " LEFT JOIN assess_task a ON t.task_id = a.id " +
                " LEFT JOIN assess_object o ON t.assess_obj_id = o.id " +
                " WHERE t.is_execute = 1" +
                " AND t.is_deleted = 0" +
                " AND a.classify_id = " + classifyId +
                " AND a.cycle_value = '" + cycleValue + "'";
    }

    public Integer continuousUnqualified(Long classifyId, String assessObj, List<String> cycleValues) {
        String sql = "select count(1) from assess_record where qualified = false and classify_id = "
                + classifyId + " and assess_obj = '" + assessObj + "' and cycle_value in  ('" + Joiner.on("','").join(cycleValues) + "')";
        return countQuery(sql, null).intValue();
    }

    public AssessRecord getById(Long id){
        String sql = " SELECT r.*, b.name AS class_name, t.name AS task_name, o.name AS assess_obj, t.cycle_type, t.cycle_value FROM assess_record r " +
                "LEFT JOIN assess_object o ON r.assess_obj_id = o.id " +
                "LEFT JOIN assess_task t ON r.task_id = t.id " +
                "LEFT JOIN assess_classify b ON t.classify_id = b.id " +
                "where r.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<AssessRecord> list = this.list(AssessRecord.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean isAllFinished(Long taskId) {
        String sql = "select count(1) " +
                "from (select count(status) total, count(status = 'FINISHED' or null) finished from assess_record where task_id = " + taskId + ") t " +
                "where t.total = t.finished and t.total > 0";
        return countQuery(sql, null).intValue() > 0;
    }
}
