package com.bocloud.cos.repository.assess;

import com.bocloud.cos.entity.assess.AssessRule;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 考核规则DAO
 *
 * @author lym
 */
@Repository
public class AssessRuleRepository extends BasicGenericDao<AssessRule, Long> {

    public AssessRuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, namedParameterJdbcTemplate, service);
    }

    public List<AssessRule> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.* from assess_rule a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(AssessRule.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from assess_rule a where a.is_deleted = 0";
        sql = getQueryBuilder().build(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<AssessRule> queryByMetricId(List<Long> metricIds) {
        String sql = "select a.* from assess_rule a where a.metric_id in (:metricIds) and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("metricIds", metricIds);
        return this.list(AssessRule.class, sql, params);
    }

    public List<AssessRule> queryByIds(List<Long> ids) {
        String sql = "select a.* from assess_rule a where a.id in (:ids) and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        return this.list(AssessRule.class, sql, params);
    }

}
