package com.bocloud.cos.driver;


import com.alibaba.fastjson.JSONObject;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.JSONTools;
import com.bocloud.cos.model.Butler;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.Iterator;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

@Slf4j
public class ButlerRepository {
    private Map<String, Butler> butlers;
    static Encryptor encryptor = new DESEncryptor();
    static ReentrantReadWriteLock rwLock = new ReentrantReadWriteLock();
    static Lock readLock;
    static Lock writeLock;

    private ButlerRepository() {
        if (null != ButlerRepository.RepositoryContainer.instance) {
            throw new RuntimeException("ButlerRepository遭受反射攻击！");
        } else {
            this.butlers = new ConcurrentHashMap();
        }
    }

    public static ButlerRepository instance() {
        return ButlerRepository.RepositoryContainer.instance;
    }

    public static ButlerRepository getInstance() {
        return ButlerRepository.RepositoryContainer.instance;
    }

    public void update(String data) {
        Assert.notNull(data, "Cloud vendor data is null");
        GeneralResult<JSONObject> result = JSONTools.isJSON(data);
        if (result.isFailed()) {
            log.error("Update config error: 传入参数格式化错误！");
        } else {
            JSONObject jsonObject = result.getData();
            Set<String> butlerKeys = jsonObject.keySet();
            writeLock.lock();

            try {
                Set<String> oldKeySet = this.butlers.keySet();
                Iterator var8 = oldKeySet.iterator();

                String key;
                while (var8.hasNext()) {
                    key = (String) var8.next();
                    if (!butlerKeys.contains(key)) {
                        this.butlers.remove(key);
                    }
                }

                var8 = butlerKeys.iterator();

                while (var8.hasNext()) {
                    key = (String) var8.next();
                    GeneralResult<Butler> butlerResult = Butler.convert(jsonObject.getString(key));
                    if (!butlerResult.isFailed()) {
                        Butler butler = butlerResult.getData();
                        this.butlers.put(key, butler);
                    }
                }
            } catch (Exception var14) {
                log.error("update config error:", var14);
            } finally {
                writeLock.unlock();
            }
        }
    }

    public void update(String key, Butler butler) {
        if (!StringUtils.isEmpty(key) && null != butler) {
            writeLock.lock();

            try {
                this.butlers.put(key, butler);
            } catch (Exception var7) {
                log.error("update key【{}】 butler exception ", key, var7);
            } finally {
                writeLock.unlock();
            }

        }
    }

    public Butler get(String uuid) {
        readLock.lock();

        Butler newButler;
        try {
            Butler butler = this.butlers.get(uuid);
            if (null == butler) {
                newButler = null;
                return newButler;
            }

            newButler = new Butler();
            BeanUtils.copyProperties(newButler, butler);
            if (Objects.isNull(butler.getCategory())) {
                newButler.setCategory("UNKNOWN");
            }

            if (!StringUtils.isEmpty(butler.getPassword())) {
                newButler.setPassword(encryptor.decrypt(butler.getPassword(), "BocloudCMP"));
            }

            if (!StringUtils.isEmpty(butler.getAccessKey())) {
                newButler.setAccessKey(encryptor.decrypt(butler.getAccessKey(), "BocloudCMP"));
            }

            if (!StringUtils.isEmpty(butler.getSecretKey())) {
                newButler.setSecretKey(encryptor.decrypt(butler.getSecretKey(), "BocloudCMP"));
            }

            Butler var4 = newButler;
            return var4;
        } catch (Exception var8) {
            log.error("get config error:", var8);
            newButler = null;
        } finally {
            readLock.unlock();
        }

        return newButler;
    }

    private static class RepositoryContainer {
        private static ButlerRepository instance = new ButlerRepository();

        private RepositoryContainer() {
        }
    }

    static {
        readLock = rwLock.readLock();
        writeLock = rwLock.writeLock();
    }
}
