package com.bocloud.cos.driver;

import com.megatron.common.model.Result;
import com.bocloud.cos.driver.utils.VendorConverter;
import com.bocloud.cos.enums.CloudProvider;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.repository.ProviderRepository;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

@Component
public class GenericDriver<T extends SuperProvider> {
    private static Logger logger = LoggerFactory.getLogger(GenericDriver.class);
    private static String PREFIX = "bocloud.cmp.";
    ButlerRepository butlerRepository = ButlerRepository.getInstance();

    public GenericDriver() {
    }

    public T getProvider(String vendor, Class<?> clazz, ProviderRepository repository, String region) {
        Butler butler = this.getButler(vendor);
        if (null == butler) {
            return null;
        } else {
            if (StringUtils.isNotEmpty(region) && !region.equalsIgnoreCase(butler.getRegionId())) {
                butler.setRegionId(region);
            }

            String provider = this.getProvider(butler);
            return (T)repository.getProvider(provider, clazz, butler);
        }
    }

    public T getProvider(String vendor, Class<?> clazz, ProviderRepository repository) {
        Butler butler = this.getButler(vendor);
        if (null == butler) {
            return null;
        } else {
            String provider = this.getProvider(butler);
            return (T)repository.getProvider(provider, clazz, butler);
        }
    }

    public Result before(T provider) {
        if (null == provider) {
            return Result.FAILED("暂不支持！");
        } else {
            return provider.accessable() ? Result.SUCCESS("success") : Result.FAILED("云平台认证失败!");
        }
    }

    public Butler getButler(String vendor) {
        Butler butler = this.butlerRepository.get(vendor);
        if (null == butler) {
            logger.error("no butler found of uuid {}", vendor);
            return null;
        } else {
            CloudProvider catalog = VendorConverter.convert(butler.getCategory());
            if (CloudProvider.UNKNOWN.equals(catalog)) {
                logger.warn("get unknown provider of {}", butler.getCategory());
                return null;
            } else {
                return butler;
            }
        }
    }

    private String getProvider(Butler butler) {
        return PREFIX + butler.getCategory().toLowerCase();
    }
}
