package com.bocloud.cos.intf;

import com.bocloud.cos.entity.ServiceAddress;
import com.bocloud.cos.params.ServiceAddressParams;
import com.bocloud.cos.entity.internel.CloudVendor;
import com.bocloud.cos.entity.internel.Department;
import com.megatron.common.model.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.util.List;

/**
 * 服务商接口
 *
 * @author hqy
 * @version 1.0
 * @since 2023年11月15日
 */
public interface ServiceAddressService {
    /**
     * 分页查询云服务
     *
     * @param pager 分页查询参数
     * @return 分页结果
     */
    GeneralResult<GridBean<ServiceAddress>> list(Pager pager, RequestContext context);

    /**
     * 删除服务商
     *
     * @param id     id
     * @param userId 操作人
     * @return 操作结果
     */
    Result remove(Long id, Long userId);

    /**
     * 服务商详情
     *
     * @param id id
     * @return 详情
     */
    GeneralResult<ServiceAddress> detail(Long id);

    /**
     * 添加服务商
     *
     * @param params 通讯录信息
     * @return 添加结果
     */
    Result create(ServiceAddressParams params, RequestContext context);

    /**
     * 编辑服务商
     *
     * @param params 编辑信息
     * @param userId 操作人
     * @return 操作结果
     */
    Result modify(ServiceAddressParams params, Long userId);

    /**
     * 查询所有平台
     *
     * @return 所有平台
     */
    GeneralResult<List<CloudVendor>> listVendors(Long excludeId);

    /**
     * 查询组织机构
     *
     * @return 组织机构列表
     */
    GeneralResult<List<Department>> listDepart();

    /**
     * 平台配置
     *
     * @param id        id
     * @param vendorIds 平台
     * @param userId    操作人
     * @return 操作结果
     */
    Result configVendor(Long id, List<Long> vendorIds, Long userId);

    /**
     * 导出
     *
     * @param params  查询参数
     * @param context 操作人
     * @return 导出信息
     */
    XSSFWorkbook export(List<Param> params, RequestContext context);
}
