package com.bocloud.cos.provider.aliyun;

import com.aliyun.bssopenapi20171214.models.QueryBillRequest;
import com.aliyun.bssopenapi20171214.models.QueryBillResponse;
import com.aliyun.bssopenapi20171214.models.QueryBillResponseBody;
import com.bocloud.cos.annotations.Provider;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.provider.BillingProvider;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Provider("bocloud.cmp.aliyun")
public class AliyunBillingProvider extends AliyunProvider implements BillingProvider {

    private BillingConvertor convertor = new BillingConvertor();

    public AliyunBillingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public GeneralResult billDetailList(String month) {
        try {
            List<PublicCloudBillModel> billModels = new ArrayList();
            QueryBillRequest request = new QueryBillRequest();
            request.setBillingCycle(month);
            request.setPageNum(1);
            request.setPageSize(100);
            QueryBillResponse response = this.getBSSClient().queryBill(request);
            QueryBillResponseBody.QueryBillResponseBodyData data = response.getBody().getData();
            Integer totalCount = data.getTotalCount();
            List<QueryBillResponseBody.QueryBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new GeneralResult(true, "暂无该月账单信息！");
            } else {
                billModels.addAll(this.convertor.convertList(items));
                int totalPage = totalCount / 100 + 1;
                if (totalPage == 1) {
                    return new GeneralResult(true, billModels, "查询用户某个账期内结算账单成功！");
                } else {
                    for (int i = 2; i <= totalPage; ++i) {
                        request.setPageNum(i);
                        request.setPageSize(100);
                        response = this.getBSSClient().queryBill(request);
                        items = response.getBody().getData().getItems().getItem();
                        billModels.addAll(this.convertor.convertList(items));
                    }

                    return new GeneralResult(true, billModels, "查询用户某个账期内结算账单成功！");
                }
            }
        } catch (Exception var10) {
            log.error("查询用户某个账期内结算账单异常：" + var10.getMessage());
            return new GeneralResult(false, "查询用户某个账期内结算账单异常：" + var10.getMessage());
        }
    }
}
