package com.bocloud.cos.provider.aliyun;

import com.aliyun.ecs20140526.Client;
import com.aliyun.teaopenapi.models.Config;
import com.aliyuncs.http.HttpClientConfig;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.megatron.common.model.GeneralResult;
import com.bocloud.cos.driver.SuperProvider;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.provider.common.AliyunLocationProvider;
import com.bocloud.cos.provider.common.LocationProvider;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

@Slf4j
public class AliyunProvider implements SuperProvider {
    private String accessKey;
    private String secretKey;
    private String regionId = "cn-hangzhou";
    private Config config;
    private Butler butler;
    private IClientProfile profile;
    private static HttpClientConfig httpClientConfig = null;

    public AliyunProvider(Butler butler) {
        this.butler = butler;
        this.accessKey = butler.getAccessKey().trim();
        this.secretKey = butler.getSecretKey().trim();
        this.regionId = butler.getRegionId() == null ? this.regionId : butler.getRegionId().trim();
        this.config = (new Config()).setAccessKeyId(this.accessKey).setAccessKeySecret(this.secretKey).setRegionId(this.regionId);
        if (!StringUtils.isEmpty(butler.getProxyUri())) {
            this.config.setHttpProxy(butler.getProxyUri());
            httpClientConfig.setHttpProxy(butler.getProxyUri());
        }

        this.profile = DefaultProfile.getProfile(this.regionId, this.accessKey, this.secretKey);
        this.profile.setHttpClientConfig(httpClientConfig);
    }

    public com.aliyun.bssopenapi20171214.Client getBSSClient() throws Exception {
        if (StringUtils.isNotEmpty(this.butler.getUrl())) {
            this.config.setEndpoint("business." + this.butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 50000;
            this.config.readTimeout = 50000;
        }

        return new com.aliyun.bssopenapi20171214.Client(this.config);
    }

    public Client getECSClient() throws Exception {
        if (StringUtils.isNotEmpty(this.butler.getUrl())) {
            this.config.setEndpoint("ecs." + this.butler.getUrl());
            this.config.setProtocol("http");
            this.config.connectTimeout = 50000;
            this.config.readTimeout = 50000;
        }

        return new Client(this.config);
    }

    @Override
    public boolean accessable() {
        try {
            LocationProvider provider = new AliyunLocationProvider(this.butler);
            GeneralResult result = provider.listRegions();
            return result.isSuccess();
        } catch (Exception var3) {
            log.error("认证失败：" + var3.getMessage());
            return false;
        }
    }
}