package com.bocloud.cos.provider.aws;

import cn.hutool.core.util.ZipUtil;
import com.alibaba.fastjson.JSON;
import com.bocloud.cos.annotations.Provider;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.model.ObjectStorageContainerModel;
import com.bocloud.cos.provider.BillingProvider;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;
import software.amazon.awssdk.services.s3.model.S3Object;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Provider("bocloud.cmp.aws")
@Slf4j
public class AwsBillingProvider extends AwsProvider implements BillingProvider {

    public AwsBillingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public GeneralResult queryInstanceBill(String month, String fileDownPath) {
        String zipPath = null;
        try {
            log.info("aws账单接受到请求参数：month:{} ", month);
            log.info("开始获取容器中......");
            AwsS3Provider awsS3Provider = new AwsS3Provider(getButler());
            GeneralResult containersResult = awsS3Provider.listContainer();
            Assert.isTrue(containersResult.isSuccess(), "aws获取容器失败！");
            List<ObjectStorageContainerModel> containerModels = JSON.parseArray(JSON.toJSONString(containersResult.getData()), ObjectStorageContainerModel.class);
            if (ListTool.isEmpty(containerModels)) {
                log.info("aws账单同步无效：没有获取到容器");
            }
            log.info("容器获取成功！");
            log.info("开始获取存储对象......");
            List<String> filePaths = new ArrayList<>();
            for (ObjectStorageContainerModel cm : containerModels) {
                String itemName = "aws-billing-detailed-line-items-with-resources-and-tags-ACTS-" + month;
                S3Object item = null;
                GeneralResult<List<S3Object>> s3ObjectResult = awsS3Provider.listItem(cm.getName());
                Assert.isTrue(s3ObjectResult.isSuccess(), "aws存储对象获取失败！");
                List<S3Object> objects = s3ObjectResult.getData();
                for (S3Object object : objects) {
                    if (object.key().contains(itemName)) {
                        item = object;
                        break;
                    }
                }
                if (null == item) {
                    log.warn("aws`{}`月的账单不存在, ObjectStorageContainerModel: {}", month, JSON.toJSONString(cm));
                    continue;
                }
                log.info("存储对象获取成功！");
                log.info("开始下载账单......");
                GeneralResult downloadResult = awsS3Provider.download(cm.getName(), item.key(), fileDownPath);
                Assert.isTrue(downloadResult.isSuccess(), "aws账单下载失败！");
                log.info("账单下载成功！");
                log.info("开始解压账单......");
                zipPath = downloadResult.getData().toString();
                ZipUtil.unzip(zipPath, fileDownPath);
                log.info("账单解压成功！");
                String outputFile = String.format("%s/%s", fileDownPath, item.key());
                String filePath = outputFile.substring(0, outputFile.length() - 4);
                log.info("底层打印文件路径:{}", filePath);
                filePaths.add(filePath);
            }
            return new GeneralResult(true, filePaths, "aws账单同步成功");
        } catch (Exception e) {
            log.error("账单同步失败: {}", e.getMessage());
            return new GeneralResult(false, "账单同步失败");
        } finally {
            // 删除压缩包临时文件
            if (null != zipPath) {
                File file = new File(zipPath);
                if (file.exists()) {
                    file.delete();
                }
            }
        }
    }
}
