package com.bocloud.cos.provider.aws;

import com.alibaba.fastjson.JSON;
import com.bocloud.cos.enums.AwsServiceEnum;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.model.ObjectStorageContainerModel;
import com.megatron.common.model.GeneralResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.s3.S3Client;
import software.amazon.awssdk.services.s3.model.*;

import java.nio.file.Paths;
import java.util.List;

public class AwsS3Provider extends AwsProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsS3Provider.class);

    private final OssConvertor convertor = new OssConvertor();

    public AwsS3Provider(Butler butler) {
        super(butler);
    }

    public GeneralResult listContainer() {
        try {
            ListBucketsResponse listBucketsResponse = ((S3Client) this.getClient(AwsServiceEnum.S3))
                    .listBuckets();
            List<Bucket> buckets = listBucketsResponse.buckets();
            List<ObjectStorageContainerModel> ossModels = convertor.convertList(buckets);
            return new GeneralResult(true, ossModels, "list Container success!");
        } catch (Exception e) {
            logger.error("list Container error :", e);
            return new GeneralResult(false, e.getMessage());
        }
    }

    public GeneralResult<List<S3Object>> listItem(String containerName) {
        try {
            ListObjectsV2Request request = ListObjectsV2Request
                    .builder()
                    .bucket(containerName)
                    .build();
            ListObjectsV2Response listObjectsV2Response = ((S3Client) this.getClient(AwsServiceEnum.S3))
                    .listObjectsV2(request);
            List<S3Object> objects = listObjectsV2Response.contents();
            return new GeneralResult<>(true, objects, "list Item success!");
        } catch (Exception e) {
            logger.error("list Item error :", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    public GeneralResult download(String containerName, String itemName, String fileDownPath) {
        try {
            String path = fileDownPath + itemName;
            GetObjectRequest request = GetObjectRequest.builder().bucket(containerName).key(itemName).build();
            S3Client client = (S3Client) this.getClient(AwsServiceEnum.S3);
            client.getObject(request, Paths.get(path));
            return new GeneralResult(true, path, "文件下载成功");
        } catch (Exception e) {
            logger.error("download error:", e);
            return new GeneralResult(false, "文件下载失败");
        }
    }
}
