package com.bocloud.cos.provider.huawei;

import com.alibaba.fastjson.JSON;
import com.bocloud.cos.annotations.Provider;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.model.HuaweiIndex;
import com.bocloud.cos.model.PublicCloudBillModel;
import com.bocloud.cos.provider.BillingProvider;
import com.bocloud.cos.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.bss.v2.BssClient;
import com.huaweicloud.sdk.bss.v2.model.*;
import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Provider("bocloud.cmp.huawei")
public class HuaweiBillingProvider extends HuaweiProvider implements BillingProvider {

    private BeanConvertor<PublicCloudBillModel, MonthlyBillRecord> convertor = new BillingConvertor();

    public HuaweiBillingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public GeneralResult<List<PublicCloudBillModel>> billDetailList(String month) {
        try {
            List<PublicCloudBillModel> bill = new ArrayList();
            int allMonth = 1;

            for (int i = 0; i <= allMonth; ++i) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
                Calendar c = Calendar.getInstance();
                c.setTime(new Date());
                c.add(2, -i);
                Date time = c.getTime();
                bill.addAll(this.bill(format.format(time)));
            }

            return new GeneralResult(true, bill, "查询账单成功");
        } catch (Exception var8) {
            var8.printStackTrace();
            log.error("查询账单失败：" + var8.getMessage());
            return new GeneralResult(false, "查询账单失败：" + var8.getMessage());
        }
    }

    public List<PublicCloudBillModel> bill(String month) {
        Map<String, String> serviceTypeMap = this.serviceTypeMap();
        Map<String, String> resourceTypeMap = this.resourceTypeMap();
        BssClient client = (BssClient) this.getClient(HuaweiIndex.service.BSS);
        ListCustomerBillsFeeRecordsRequest request = new ListCustomerBillsFeeRecordsRequest();
        request.withBillCycle(month);
        ListCustomerBillsFeeRecordsResponse response = client.listCustomerBillsFeeRecords(request);
        Integer totalCount = response.getTotalCount();
        Integer limit = 1000;
        Integer totalPage = totalCount % limit == 0 ? totalCount / limit : totalCount / limit + 1;
        List<MonthlyBillRecord> billRecordList = new ArrayList(8);

        for (int i = 0; i <= totalPage; ++i) {
            request.withBillCycle(month).withOffset(i * limit).withLimit(limit);
            response = client.listCustomerBillsFeeRecords(request);
            billRecordList.addAll(response.getRecords());
        }

        log.info("start sync : " + month);
        if (!ListTool.isEmpty(billRecordList)) {
            log.info("record : " + JSON.toJSONString(billRecordList.get(0)));
        }

        List<PublicCloudBillModel> publicCloudBillModels = this.convertor.convertList(billRecordList);
        log.info("start convertor : " + month);
        Iterator var12 = publicCloudBillModels.iterator();

        while (var12.hasNext()) {
            PublicCloudBillModel model = (PublicCloudBillModel) var12.next();
            if (null != model.getProductCode()) {
                model.setProductName(serviceTypeMap.get(model.getProductCode()));
            }

            if (null != model.getResourceId()) {
                model.setProductDetail(resourceTypeMap.get(model.getResourceId()));
            }
        }

        if (!ListTool.isEmpty(publicCloudBillModels)) {
            System.out.println(JSON.toJSONString(publicCloudBillModels.get(0)));
        }

        return publicCloudBillModels;
    }

    public Map<String, String> serviceTypeMap() {
        try {
            Map<String, String> serviceTypeMap = new HashMap(8);
            BssClient client = (BssClient) this.getClient(HuaweiIndex.service.BSS);
            ListServiceTypesRequest request = new ListServiceTypesRequest();
            ListServiceTypesResponse response = client.listServiceTypes(request);
            List<ServiceType> serviceTypes = response.getServiceTypes();
            if (!ListTool.isEmpty(serviceTypes)) {
                Iterator var6 = serviceTypes.iterator();

                while (var6.hasNext()) {
                    ServiceType serviceType = (ServiceType) var6.next();
                    serviceTypeMap.put(serviceType.getServiceTypeCode(), serviceType.getServiceTypeName());
                }
            }

            return serviceTypeMap;
        } catch (Exception var8) {
            var8.printStackTrace();
            return null;
        }
    }

    public Map<String, String> resourceTypeMap() {
        try {
            Map<String, String> resourceTypeMap = new HashMap(8);
            BssClient client = (BssClient) this.getClient(HuaweiIndex.service.BSS);
            ListResourceTypesRequest request = new ListResourceTypesRequest();
            ListResourceTypesResponse response = client.listResourceTypes(request);
            List<ResourceType> resourceTypes = response.getResourceTypes();
            if (!ListTool.isEmpty(resourceTypes)) {
                Iterator var6 = resourceTypes.iterator();

                while (var6.hasNext()) {
                    ResourceType resourceType = (ResourceType) var6.next();
                    resourceTypeMap.put(resourceType.getResourceTypeCode(), resourceType.getResourceTypeName());
                }
            }

            return resourceTypeMap;
        } catch (Exception var8) {
            var8.printStackTrace();
            return null;
        }
    }
}
