package com.bocloud.cos.provider.tencent;

import com.bocloud.cos.annotations.Provider;
import com.bocloud.cos.enums.TencentEnum;
import com.bocloud.cos.model.Butler;
import com.bocloud.cos.provider.BillingProvider;
import com.megatron.common.model.GeneralResult;
import com.tencentcloudapi.billing.v20180709.BillingClient;
import com.tencentcloudapi.billing.v20180709.models.BillResourceSummary;
import com.tencentcloudapi.billing.v20180709.models.DescribeBillResourceSummaryRequest;
import com.tencentcloudapi.billing.v20180709.models.DescribeBillResourceSummaryResponse;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Arrays;

@Slf4j
@Provider("bocloud.cmp.tencent")
public class TencentBillingProvider extends TencentProvider implements BillingProvider {

    private BillingConvertor convertor = new BillingConvertor();

    public TencentBillingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public GeneralResult billDetailList(String month) {
        try {
            DescribeBillResourceSummaryRequest req = new DescribeBillResourceSummaryRequest();
            req.setPeriodType("byPayTime");
            req.setMonth(month);
            BillingClient billClient = (BillingClient) this.getClient(TencentEnum.BILLING);
            Integer offset = 0;
            ArrayList summarys = new ArrayList();
            while (true) {
                while (offset >= 0) {
                    req.setOffset(offset * 100);
                    req.setLimit(100);
                    DescribeBillResourceSummaryResponse resp = billClient.DescribeBillResourceSummary(req);
                    BillResourceSummary[] resourceSummarySet = resp.getResourceSummarySet();
                    if (null != resourceSummarySet && resourceSummarySet.length > 0) {
                        summarys.addAll(Arrays.asList(resourceSummarySet));
                        offset = offset + 1;
                    } else {
                        offset = -1;
                    }
                }
                return new GeneralResult(true, this.convertor.convertList(summarys), "查询成功！");
            }
        } catch (Exception var8) {
            log.error("查询失败: {}" + var8);
            return new GeneralResult(false, "查询失败 ！" + var8.getMessage());
        }
    }
}
