package com.bocloud.cos.provider.tencent;


import com.bocloud.cos.driver.SuperProvider;
import com.bocloud.cos.enums.TencentEnum;
import com.bocloud.cos.model.Butler;
import com.tencentcloudapi.billing.v20180709.BillingClient;
import com.tencentcloudapi.cbs.v20170312.CbsClient;
import com.tencentcloudapi.clb.v20180317.ClbClient;
import com.tencentcloudapi.common.AbstractClient;
import com.tencentcloudapi.common.Credential;
import com.tencentcloudapi.common.profile.ClientProfile;
import com.tencentcloudapi.common.profile.HttpProfile;
import com.tencentcloudapi.cvm.v20170312.CvmClient;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesRequest;
import com.tencentcloudapi.cvm.v20170312.models.DescribeInstancesResponse;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import org.apache.commons.lang3.StringUtils;

public class TencentProvider implements SuperProvider {

    private String secretId;
    private String secretKey;
    private Credential credential;
    private String regionId = "ap-beijing";
    private ClientProfile clientProfile = new ClientProfile();
    private HttpProfile httpProfile = new HttpProfile();

    public TencentProvider(Butler butler) {
        this.secretId = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.regionId = butler.getRegionId();
        this.credential = new Credential(this.secretId, this.secretKey);
    }

    @Override
    public boolean accessable() {
        try {
            CvmClient client = (CvmClient) this.getClient(TencentEnum.CVM);
            DescribeInstancesRequest req = new DescribeInstancesRequest();
            DescribeInstancesResponse response = client.DescribeInstances(req);
            response.getInstanceSet();
            return true;
        } catch (Exception var4) {
            return false;
        }
    }

    public AbstractClient getClient(TencentEnum type) {
        AbstractClient abstractClient = null;
        this.configuration(type);
        switch (type) {
            case CVM:
                abstractClient = new CvmClient(this.credential, this.regionId, this.clientProfile);
                break;
            case CBS:
                abstractClient = new CbsClient(this.credential, this.regionId, this.clientProfile);
                break;
            case VPC:
                abstractClient = new VpcClient(this.credential, this.regionId, this.clientProfile);
                break;
            case CLB:
                abstractClient = new ClbClient(this.credential, this.regionId, this.clientProfile);
                break;
            case BILLING:
                abstractClient = new BillingClient(this.credential, this.regionId, this.clientProfile);
        }

        return abstractClient;
    }

    private void configuration(TencentEnum type) {
        this.httpProfile.setConnTimeout(600);
        this.clientProfile.setHttpProfile(this.httpProfile);
        if (StringUtils.isBlank(this.regionId)) {
            this.regionId = "ap-beijing";
        }

        this.httpProfile.setEndpoint(type.toString().toLowerCase() + "." + this.regionId + ".tencentcloudapi.com");
    }
}
