package com.bocloud.cos.service;

import com.bocloud.cos.entity.Attachment;
import com.bocloud.cos.repository.AttachmentRepository;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.UUID;

@Slf4j
@Service
public class AttachmentService {

    @Value("${spring.upload.path:/home/cmp}")
    private String path;

    @Autowired
    private AttachmentRepository attachmentRepository;

    public void saveAttachment(Attachment attachment) {
        if (null != attachment.getId() && 0L != attachment.getId()) {
            Attachment att = attachmentRepository.get(Attachment.class, attachment.getId());
            if (StringUtils.isBlank(att.getGroupId())) {
                attachmentRepository.update(attachment);
            } else {
                attachment.setId(null);
                attachmentRepository.save(attachment);
            }
        } else {
            attachmentRepository.save(attachment);
        }
    }

    public Attachment get(Long id) {
        return attachmentRepository.get(Attachment.class, id);
    }

    public Attachment upload(MultipartFile multipartFile, RequestContext context) throws Exception {
        if (multipartFile == null) {
            return null;
        }
        File file = null;
        Attachment attachment = new Attachment();
        try {
            String extname = getExtname(multipartFile.getOriginalFilename());
            String filename = File.separator + UUID.randomUUID().toString().replaceAll("-", "") + (null == extname ? "" : "." + extname);
            file = new File("attachment" + filename);
            File destFile = new File(path, file.getPath());
            attachment.setContentType(multipartFile.getContentType());
            attachment.setSize(multipartFile.getSize());
            attachment.setName(multipartFile.getOriginalFilename());
            attachment.setPath(file.getPath());
            if (!destFile.getParentFile().exists()) {
                destFile.getParentFile().mkdirs();
            }
            multipartFile.transferTo(destFile);

            attachment.setCreatorId(context.getTarget());
            attachment.setGmtCreate(new Date());
            saveAttachment(attachment);
        } catch (IOException e) {
            // 删除垃圾文件
            if (null != file) {
                if (new File(path, file.getPath()).exists()) {
                    boolean flag = new File(path, file.getPath()).delete();
                    log.info("临时文件删除状态：{}", flag);
                }
            }
            throw new Exception(e);
        }
        return attachment;
    }

    /**
     * 获取文件扩展名
     *
     * @param filename
     * @return
     */
    private String getExtname(String filename) {
        int index = filename.lastIndexOf('.');
        if (index == -1) {
            return null;
        }
        return filename.substring(index + 1);
    }

    public void delete(Long id) {
        Attachment attachment = attachmentRepository.get(Attachment.class, id);
        boolean flag = attachmentRepository.delete(Attachment.class, id);
        if (flag) {
            new File(path, attachment.getPath()).delete();
        }
    }
}
