package com.bocloud.cos.service;

import com.bocloud.cos.entity.CostCenter;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.intf.CostCenterService;
import com.bocloud.cos.repository.CostCenterRepository;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/12/30 14:38
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class CostCenterServiceImpl implements CostCenterService {

    @Autowired
    private CostCenterRepository costCenterRepository;

    @Autowired
    private  LockFactory lockFactory;

    @Override
    public GeneralResult<GridBean<CostCenter>> list(Pager pager) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            pager.setSorter(sorter);
            int total = this.costCenterRepository.count(params);
            List<CostCenter> list = this.costCenterRepository.list(pager);
            GridBean<CostCenter> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query service cost center list error:{}", e.getMessage());
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    @Override
    public GeneralResult<CostCenter> detail(Long id) {
         CostCenter costCenter = this.costCenterRepository.get(CostCenter.class,id);
         Assert.isTrue(null != costCenter,"数据不存在");
         return new GeneralResult<>(true, costCenter, "查询成功");
    }

    @Override
    public Result create(CostCenter costCenter, RequestContext context) {
        CostCenter byCode = costCenterRepository.getByCode(costCenter.getCode());
        Assert.isTrue(null == byCode,"编码已存在");
        costCenter.setDeleted(false);
        costCenter.setGmtCreate(new Date());
        costCenter.setModifyId(context.getTarget());
        this.costCenterRepository.save(costCenter);
        return new GeneralResult<>(true,"创建成功");
    }

    @Override
    public Result modify(Long id, CostCenter costCenter, RequestContext context) {
        String path = CostCenter.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CostCenter byId = costCenterRepository.get(CostCenter.class, id);
            Assert.isTrue(null != byId, "数据不存在!");
            byId.setGmtModify(new Date());
            byId.setName(costCenter.getName());
            byId.setRemark(costCenter.getRemark());
            costCenterRepository.update(byId);
            return new GeneralResult<>(true,"修改成功!");
        } catch (Exception e) {
            log.error("modify costCenter error:", e);
            return new Result(false, "修改成功");
        }
    }

    @Override
    public Result remove(Long id, RequestContext context) {
        String path = CostCenter.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CostCenter costCenter = costCenterRepository.get(CostCenter.class,id);
            Assert.notNull(costCenter, "数据不存在！");
            costCenter.setDeleted(true);
            costCenter.setModifyId(context.getTarget());
            costCenterRepository.update(costCenter);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove costCenter error:", e);
            return new Result(false, "删除失败");
        }
    }
}
