package com.bocloud.cos.service;

import com.bocloud.cos.entity.MetaQuota;
import com.bocloud.cos.repository.MetaQuotaRepository;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
@RequiredArgsConstructor
public class MetaQuotaService {

    private final LockFactory lockFactory;
    private final MetaQuotaRepository metaQuotaRepository;

    public GridBean<MetaQuota> list(Pager pager) {
        pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
        pager.getSorter().put("gmtCreate", Common.ONE);
        int total = metaQuotaRepository.count(pager.getParams());
        List<MetaQuota> list = metaQuotaRepository.list(pager);
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    public void create(MetaQuota domain, RequestContext context) {
        domain.setCreatorId(context.getTarget());
        metaQuotaRepository.save(domain);
    }

    public void modify(MetaQuota domain, RequestContext context) {
        domain.setMenderId(context.getTarget());
        metaQuotaRepository.update(domain);
    }

    public Result remove(Long id, RequestContext context) {
        String path = MetaQuota.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            MetaQuota metaQuota = metaQuotaRepository.query(id);
            Assert.notNull(metaQuota, "数据不存在！");
            metaQuota.setMenderId(context.getTarget());
            metaQuota.setDeleted(true);
            metaQuotaRepository.update(metaQuota);
            return new Result(true, "删除成功");
        }
    }

    public GeneralResult<MetaQuota> detail(Long id) {
        MetaQuota metaQuota = metaQuotaRepository.query(id);
        Assert.notNull(metaQuota, "数据不存在！");
        return new GeneralResult<>(true, metaQuota, "查询成功");
    }
}
