package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessCollectionMetric;
import com.bocloud.cos.entity.assess.AssessCollectionMetricDetail;
import com.bocloud.cos.repository.assess.AssessCollectionMetricDetailRepository;
import com.bocloud.cos.repository.assess.AssessCollectionMetricRepository;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 考核采集指标业务逻辑处理
 *
 * @author lym
 */
@Service
public class AssessCollectionMetricService {

    @Autowired
    private AssessCollectionMetricRepository collectionMetricRepository;

    @Autowired
    private AssessCollectionMetricDetailRepository collectionMetricDetailRepository;

    public GridBean<AssessCollectionMetric> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        int total = collectionMetricRepository.count(pager.getParams());
        List<AssessCollectionMetric> list = collectionMetricRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams());
        list.forEach(metric -> {
            List<AssessCollectionMetricDetail> assessCollectionMetricDetails = collectionMetricDetailRepository.listByMetricId(metric.getId());
            metric.setMetricDetails(assessCollectionMetricDetails);
        });
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    @Transactional(rollbackFor = Exception.class)
    public void save(AssessCollectionMetric collectionMetric, RequestContext context) {
        Assert.notEmpty(collectionMetric.getMetricDetails(), "请选择关联监控指标项");
        saveOf(collectionMetric, context);
        collectionMetricRepository.save(collectionMetric);

        batchSaveMetricDetail(collectionMetric.getId(), collectionMetric.getMetricDetails());
    }

    private void batchSaveMetricDetail(Long metricId, List<AssessCollectionMetricDetail> metricDetails) {
        metricDetails.stream().forEach(metricDetail -> {
            metricDetail.setCollectionMetricId(metricId);
        });
        collectionMetricDetailRepository.batchSave(metricDetails, metricDetails.size());
    }

    public void update(AssessCollectionMetric collectionMetric, RequestContext context) {
        AssessCollectionMetric ac = collectionMetricRepository.get(AssessCollectionMetric.class, collectionMetric.getId());
        ac.setName(collectionMetric.getName());
        ac.setRemark(collectionMetric.getRemark());
        updateOf(ac, context);
        boolean update = collectionMetricRepository.update(ac);
        if (update) {
            collectionMetricDetailRepository.deleteByMetricId(ac.getId());
            batchSaveMetricDetail(ac.getId(), collectionMetric.getMetricDetails());
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public void remove(Long id) {
        collectionMetricDetailRepository.deleteByMetricId(id);
        collectionMetricRepository.deleteById(id);
    }

    private void saveOf(AssessCollectionMetric collectionMetric, RequestContext context) {
        collectionMetric.setCreatorId(context.getTarget());
        collectionMetric.setGmtCreate(new Date());
    }

    private void updateOf(AssessCollectionMetric collectionMetric, RequestContext context) {
        collectionMetric.setMenderId(context.getTarget());
        collectionMetric.setGmtModify(new Date());
    }
}
