package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessConformityStandard;
import com.bocloud.cos.repository.assess.AssessConformityStandardRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * @author wpj
 * @since 2023/10/27
 */
@Slf4j
@Service
public class AssessConformityStandardService {
    @Autowired
    private AssessConformityStandardRepository conformityStandardRepository;


    public GeneralResult save(AssessConformityStandard assessConformityStandard, RequestContext context) {
        int nameCount = conformityStandardRepository.duplicateName(assessConformityStandard.getName());
        if (nameCount >= 1) {
            return new GeneralResult(false, "名称重复，请重新修改");
        }
        assessConformityStandard.setCreatorId(context.getTarget());
        try {
            conformityStandardRepository.save(assessConformityStandard);
            return new GeneralResult(true, "新增合格标准成功");
        }catch (Exception e) {
            return new GeneralResult(false, "新增合格标准失败");
        }
    }


    public GridBean<AssessConformityStandard> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        pager.getSorter().put("gmtCreate", Common.ONE);
        int total = conformityStandardRepository.count(pager.getParams());
        List<AssessConformityStandard> list = conformityStandardRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams());
        list.forEach(assessConformityStandard -> {
            assessConformityStandard.setStandardRangeName(assessConformityStandard.getStandardRange().getName());
        });
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }


    public GeneralResult update(AssessConformityStandard assessConformityStandard, RequestContext context) {
        int nameCount = conformityStandardRepository.duplicateName(assessConformityStandard.getName());
        if (nameCount > 1) {
            return new GeneralResult(false, "名称重复，请重新修改");
        }
        AssessConformityStandard assessConformityStandardOld = getConformityStandard(assessConformityStandard.getId());
        BeanUtils.copyProperties(assessConformityStandard, assessConformityStandardOld,null,"gmtCreate","creatorId");
        updateOf(assessConformityStandardOld, context);
        assessConformityStandardOld.setDeleted(false);
        conformityStandardRepository.update(assessConformityStandardOld);
        return new GeneralResult(true, "编辑合格标准成功");
    }

    public void delete(Long id, RequestContext context) {
        AssessConformityStandard assessConformityStandard = getConformityStandard(id);
        assessConformityStandard.setDeleted(true);
        updateOf(assessConformityStandard, context);
        conformityStandardRepository.update(assessConformityStandard);
    }

    public AssessConformityStandard getConformityStandard(Long id) {
        AssessConformityStandard assessConformityStandard = conformityStandardRepository.get(AssessConformityStandard.class, id);
        Assert.notNull(assessConformityStandard, "该合格标准不存在");
        assessConformityStandard.setStandardRangeName(assessConformityStandard.getStandardRange().getName());
        return assessConformityStandard;
    }

    private void updateOf(AssessConformityStandard assessConformityStandard, RequestContext context) {
        assessConformityStandard.setMenderId(context.getTarget());
        assessConformityStandard.setGmtModify(new Date());
    }

    public AssessConformityStandard getByTaskId(Long taskId) {
        return conformityStandardRepository.getByTaskId(taskId);
    }

}