package com.bocloud.cos.service.assess;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cos.entity.assess.AssessLevel;
import com.bocloud.cos.repository.assess.AssessLevelRepository;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;

import lombok.extern.slf4j.Slf4j;

/**
 * 评价等级业务逻辑处理
 */
@Slf4j
@Service
public class AssessLevelService {

	@Autowired
	private AssessLevelRepository assessLevelRepository;

	public void save(AssessLevel level, RequestContext context) {
		saveOf(level, context);
		assessLevelRepository.save(level);
	}

	public GridBean<AssessLevel> list(Pager pager, RequestContext context) {
		pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
		pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
		pager.getSorter().put("gmtCreate", Common.ONE);
		int total = assessLevelRepository.count(pager.getParams());
		List<AssessLevel> list = assessLevelRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(),
				pager.getParams());
		return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
	}

	public void update(AssessLevel level, RequestContext context) {
		AssessLevel al = getAssessLevel(level.getId());
		al.setName(level.getName());
		al.setLevels(level.getLevels());
		al.setRemark(level.getRemark());
		updateOf(al, context);
		assessLevelRepository.update(al);
	}

	public void remove(Long id, RequestContext context) {
		AssessLevel level = getAssessLevel(id);
		level.setDeleted(true);
		updateOf(level, context);
		assessLevelRepository.update(level);
	}

	private AssessLevel getAssessLevel(Long id) {
		AssessLevel level = assessLevelRepository.get(AssessLevel.class, id);
		Assert.notNull(level, "该评价等级不存在");
		return level;
	}

	private void saveOf(AssessLevel level, RequestContext context) {
		level.setCreatorId(context.getTarget());
		level.setGmtCreate(new Date());
	}

	private void updateOf(AssessLevel level, RequestContext context) {
		level.setMenderId(context.getTarget());
		level.setGmtModify(new Date());
	}

	public AssessLevel query(Long id) {
		return getAssessLevel(id);
	}
}
