package com.bocloud.cos.service.assess;

import com.bocloud.cos.entity.assess.AssessMetric;
import com.bocloud.cos.repository.assess.AssessMetricRepository;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 考核指标业务逻辑处理
 *
 * @author lym
 */
@Slf4j
@Service
public class AssessMetricService {

    @Autowired
    private AssessMetricRepository assessMetricRepository;

    public GridBean<AssessMetric> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        pager.getSorter().put("gmtCreate", Common.ONE);
        int total = assessMetricRepository.count(pager.getParams());
        List<AssessMetric> list = assessMetricRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams());
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    public void save(AssessMetric assessMetric, RequestContext context) {
        saveOf(assessMetric, context);
        assessMetricRepository.save(assessMetric);
    }

    public void update(AssessMetric assessMetric, RequestContext context) {
        AssessMetric ai = getAssessMetric(assessMetric.getId());
        ai.setName(assessMetric.getName());
        ai.setAcquisitionMode(assessMetric.getAcquisitionMode());
        ai.setCollectionObjId(assessMetric.getCollectionObjId());
        ai.setCollectionMetricId(assessMetric.getCollectionMetricId());
        updateOf(ai, context);
        assessMetricRepository.update(ai);
    }

    public void remove(Long id, RequestContext context) {
        assessMetricRepository.remove(AssessMetric.class, id);
    }

    private AssessMetric getAssessMetric(Long id) {
        AssessMetric assessMetric = assessMetricRepository.get(AssessMetric.class, id);
        Assert.notNull(assessMetric, "该考核指标不存在");
        return assessMetric;
    }

    private void saveOf(AssessMetric assessMetric, RequestContext context) {
        assessMetric.setCreatorId(context.getTarget());
        assessMetric.setGmtCreate(new Date());
        assessMetric.setDeleted(false);
    }

    private void updateOf(AssessMetric assessMetric, RequestContext context) {
        assessMetric.setMenderId(context.getTarget());
        assessMetric.setGmtModify(new Date());
        assessMetric.setDeleted(false);
    }

    public List<AssessMetric> queryNotBindTask(Long classifyId, Long taskId) {
        return assessMetricRepository.queryNotBindTaskByClassifyAndTaskId(classifyId, taskId);
    }

    public List<AssessMetric> queryByTaskId(Long taskId) {
        return assessMetricRepository.queryByTaskId(taskId);
    }

}
