package com.bocloud.cos.service.assess;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.excel.util.MapUtils;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cos.entity.assess.*;
import com.bocloud.cos.enums.AssessAcquisitionModeEnum;
import com.bocloud.cos.enums.AssessRecordStatusEnum;
import com.bocloud.cos.enums.AssessTaskStatusEnum;
import com.bocloud.cos.service.internal.CmsInternalService;
import com.bocloud.cos.service.internal.ImsInternalService;
import com.bocloud.cos.service.internal.SmsProjectService;
import com.bocloud.cos.service.internal.SmsUserService;
import com.bocloud.cos.model.AssessScoreProportionModel;
import com.bocloud.cos.repository.ServiceAddressVendorRepository;
import com.bocloud.cos.repository.assess.*;
import com.bocloud.cos.service.ResourceEventPublisher;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.entity.internel.CloudServer;
import com.bocloud.cos.entity.internel.Project;
import com.bocloud.cos.entity.internel.User;
import com.google.common.base.Joiner;
import com.google.common.util.concurrent.AtomicDouble;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.text.MessageFormat;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.time.ZoneOffset;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 考核记录业务逻辑处理
 *
 * @author lym
 */
@Slf4j
@Service
public class AssessRecordService {

    @Autowired
    AssessTaskRepository taskRepository;
    @Autowired
    private AssessRecordRepository assessRecordRepository;
    @Autowired
    private AssessRecordMetricRepository recordMetricRepository;
    @Autowired
    private ServiceAddressVendorRepository serviceAddressVendorRepository;
    @Autowired
    private AssessConformityStandardService conformityStandardService;
    @Autowired
    private AssessMetricRepository metricRepository;
    @Autowired
    private AssessRuleRepository ruleRepository;
    @Autowired
    private AssessCollectionMetricDetailRepository collectionMetricDetailRepository;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private SmsUserService smsUserService;
    @Autowired
    private SmsProjectService projectService;
    @Autowired
    private CmsInternalService cmsInternalService;
    @Autowired
    private ImsInternalService imsInternalService;
    @Autowired
    ResourceEventPublisher resourceEventPublisher;

    /**
     * 站内消息队列
     */
    private static final String NOTICE_QUEUE = "event.notice.queue";

    public GridBean<AssessRecord> list(Pager pager, RequestContext context) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        pager.getSorter().put("gmtCreate", Common.ONE);

        boolean isManager = true;
        GeneralResult<User> userResult = smsUserService.detail(context.getTarget());
        if (userResult.isSuccess()) {
            isManager = smsUserService.detail(context.getTarget()).getData().getIsManager();
        }

        int total = assessRecordRepository.count(pager.getParams(), isManager, context.getTarget());
        List<AssessRecord> list = assessRecordRepository.list(pager.getPage(), pager.getRows(), pager.getSorter(), pager.getParams(), isManager, context.getTarget());

        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    public void receiveAssess(Long recordId, RequestContext context) {
        AssessRecord record = assessRecordRepository.get(AssessRecord.class, recordId);
        updateRecordStatus(record, AssessRecordStatusEnum.SUBMISSION, context);
    }

    private void updateRecordStatus(AssessRecord record, AssessRecordStatusEnum recordStatusEnum, RequestContext context) {
        record.setStatus(recordStatusEnum.getCode());
        updateOf(record, context);
        assessRecordRepository.update(record);
    }

    private void updateOf(AssessRecord assessRecord, RequestContext context) {
        assessRecord.setMenderId(context.getTarget());
        assessRecord.setGmtModify(new Date());
    }

    public List<AssessRecordMetric> metricDetail(Long recordId) {
        return recordMetricRepository.queryByRecordId(recordId);
    }

    @Async
    public void updateScore(List<AssessRecordMetric> list, RequestContext context) {
        list.forEach(atii -> {
            atii.setScore(atii.getScore() == null ? 0.0 : atii.getScore());
            atii.setMenderId(context.getTarget());
            atii.setGmtModify(new Date());
        });
        recordMetricRepository.batchUpdate(list, list.size());

        AssessRecord assessRecord = assessRecordRepository.getById(list.get(0).getRecordId());
        assessScoring(assessRecord, context);
    }

    public boolean updateAttachmentId(Long recordMetricId, Long attachmentId, RequestContext context) {
        AssessRecordMetric arm = recordMetricRepository.get(AssessRecordMetric.class, recordMetricId);
        arm.setAttachmentId(attachmentId);
        arm.setMenderId(context.getTarget());
        arm.setGmtModify(new Date());
        return recordMetricRepository.update(arm);
    }

    public void submitAssess(Long recordId, RequestContext context) {
        // 提交考核任务后，修改状态为考核评分中，并开始自动评分
        AssessRecord record = assessRecordRepository.getById(recordId);
        updateRecordStatus(record, AssessRecordStatusEnum.ASSESSING, context);

        List<AssessRecordMetric> recordMetrics = recordMetricRepository.queryByRecordId(record.getId());
        List<AssessRecordMetric> recordMetricsOfAuto = recordMetrics.stream()
                .filter(recordMetric -> AssessAcquisitionModeEnum.AUTO.getCode().equals(recordMetric.getAcquisitionMode()))
                .collect(Collectors.toList());

        // 自动评分
        automaticAcquisitionMark(record, recordMetricsOfAuto, record.getCycleValue());

        // 如果都是自动评分则直接开始考核打分
        if (recordMetrics.size() == recordMetricsOfAuto.size()) {
            assessScoring(record, context);
        }
    }

    @Async
    public void assessScoring(AssessRecord record, RequestContext context) {
        record.setGmtStart(new Date());
        boolean success = true;
        String msgContent = String.format("任务【%s】下的考核对象【%s】完成考核", record.getTaskName(), record.getAssessObj());
        try {
            log.info("开始给ID为[{}]的考核记录评分", record.getId());
            List<AssessRecordMetric> recordMetrics = recordMetricRepository.queryByRecordId(record.getId());

            double totalScore = recordMetrics.stream()
                    .mapToDouble(assessTaskGroupMetricInfo -> BigDecimal.valueOf(assessTaskGroupMetricInfo.getScore())
                            .multiply(BigDecimal.valueOf(assessTaskGroupMetricInfo.getWeight()))
                            .divide(BigDecimal.valueOf(assessTaskGroupMetricInfo.getMaxScore()), 5, RoundingMode.HALF_UP)
                            .doubleValue())
                    .sum();

            int qualifiedCount = (int) recordMetrics.stream()
                    .filter(AssessRecordMetric::getQualified)
                    .count();

            int unqualifiedCount = (int) recordMetrics.stream()
                    .filter(recordMetric -> !recordMetric.getQualified())
                    .count();

            AssessConformityStandard conformityStandard = conformityStandardService.getByTaskId(record.getTaskId());
            String calculatedValue = getCalculatedValue(conformityStandard, totalScore, qualifiedCount);

            String performanceLevel = calculatePerformanceLevel(calculatedValue, conformityStandard);

            record.setExecute(true);
            record.setExecuteScore(totalScore);
            record.setExecuteResult("考核成功");
            record.setExecutorId(context.getTarget());
            record.setStatus(AssessRecordStatusEnum.FINISHED.getCode());
            record.setQualifiedCount(qualifiedCount);
            record.setUnqualifiedCount(unqualifiedCount);
            record.setPerformanceLevel(performanceLevel);
            record.setGmtEnd(new Date());
            updateOf(record, context);
        } catch (Exception e) {
            log.error("ID为[{}]的任务执行失败：{}", record.getId(), e);
            success = false;
            msgContent = msgContent.replace("考核完成", "考核失败：" + e.getMessage());
            record.setExecuteResult("考核失败：" + e.getMessage());
        } finally {
            updateOf(record, context);
            record.setGmtEnd(new Date());
            assessRecordRepository.update(record);
            log.info("ID为[{}]的考核记录评分完成", record.getId());

            resourceEventPublisher.send(new OperateResult(success, msgContent, "assess", null,OperateResult.OperateCategory.ResourceEvent, context));

            boolean allFinished = assessRecordRepository.isAllFinished(record.getTaskId());
            if (allFinished) {
                taskRepository.updateStatus(record.getTaskId(), AssessTaskStatusEnum.FINISHED.getCode());
            }
        }
    }

    private String getCalculatedValue(AssessConformityStandard conformityStandard, double totalScore, int qualifiedCount) {
        String calculatedValue = "0";
        switch (conformityStandard.getStandardRange()) {
            case TOTAL:
                calculatedValue = String.valueOf(totalScore);
                break;
            case SINGLE:
                calculatedValue = String.valueOf(qualifiedCount);
                break;
            default:
                break;
        }
        return calculatedValue;
    }

    private String calculatePerformanceLevel(String calculatedValue, AssessConformityStandard conformityStandard) {
        JSONObject jsonObject = JSON.parseObject(conformityStandard.getRangeConf());
        List<AssessConformityStandard.StandardInterval> standardIntervals = JSON.parseArray(jsonObject.getString("score"), AssessConformityStandard.StandardInterval.class);
        for (AssessConformityStandard.StandardInterval standardInterval : standardIntervals) {
            String condition = calculatedValue + standardInterval.getOperator() + standardInterval.getValue() + " && " + calculatedValue + standardInterval.getOperator2() + standardInterval.getValue2();
            if (executeCondition(condition)) {
                return standardInterval.getName();
            }
        }
        return null;
    }

    /**
     * 自动采集指标
     *
     * @param record        考核记录（即考核对象的考核任务）
     * @param recordMetrics 考核记录指标明细
     * @param cycleValue    考核周期值
     */
    private void automaticAcquisitionMark(AssessRecord record, List<AssessRecordMetric> recordMetrics, String cycleValue) {
        if (recordMetrics.isEmpty()) {
            return;
        }

        switch (record.getClassName()) {
            case "云服务商":
                List<Long> vendorIds = serviceAddressVendorRepository.listVendorIdByName(record.getAssessObj());
                if (!vendorIds.isEmpty()) {
                    List<Param> vendorParams = Collections.singletonList(new Param(MapTools.simpleMap("vendorId", Joiner.on(",").join(vendorIds)), Sign.IN));
                    processAssessRecords(record, recordMetrics, vendorParams, cycleValue);
                }
                break;
            case "云使用方":
                List<Project> projects = projectService.listByDepart(record.getAssessObj());
                if (!projects.isEmpty()) {
                    List<Long> projectIds = projects.stream().map(Project::getId).collect(Collectors.toList());
                    List<Param> projectParams = Collections.singletonList(new Param(MapTools.simpleMap("projectIds", projectIds), Sign.IN));
                    processAssessRecords(record, recordMetrics, projectParams, cycleValue);
                }
                break;
            case "项目考核":
                Project project = projectService.listByProjectName(record.getAssessObj());
                if (project != null) {
                    List<Param> projectParams = Collections.singletonList(new Param(MapTools.simpleMap("projectId", project.getId()), Sign.EQ));
                    processAssessRecords(record, recordMetrics, projectParams, cycleValue);
                }
                break;
            default:
                break;
        }

        recordMetricRepository.batchUpdate(recordMetrics, recordMetrics.size());
    }

    private void processAssessRecords(AssessRecord record, List<AssessRecordMetric> recordMetrics, List<Param> params, String cycleValue) {
        List<CloudServer> cloudServers = imsInternalService.queryByParams(params);
        log.info("考核分类：{}，考核对象: {} 下的云主机有{}个：{}",
                record.getClassName(), record.getAssessObj(), cloudServers.size(), JSON.toJSONString(cloudServers));
        if (cloudServers.isEmpty()) {
            return;
        }

        Map<String, String> dayAndTimestamp = getDayAndTimestamp(record.getCycleType(), cycleValue);
        int dayOfMonth = Integer.parseInt(dayAndTimestamp.get("dayOfMonth"));
        String timeStamp = dayAndTimestamp.get("timestamp");

        for (AssessRecordMetric recordMetric : recordMetrics) {
            List<AssessCollectionMetricDetail> collectionMetricDetails = collectionMetricDetailRepository.listByMetricRuleId(recordMetric.getMetricRuleId());
            AssessRule rule = ruleRepository.get(AssessRule.class, recordMetric.getMetricRuleId());

            double totalUseRate = 0.0;
            for (CloudServer cloudServer : cloudServers) {
                for (AssessCollectionMetricDetail acmd : collectionMetricDetails) {
                    if (acmd.getResourceType().contains(cloudServer.getVendorType())) {
                        String expr = String.format("round((%s(%s{instance_id=\"%s\"}[%dd})),0.01)",
                                rule.getDataFunction(), acmd.getExpression(), cloudServer.getInstanceId(), dayOfMonth);
                        log.info("===> 监控指标表达式：{}", expr);
                        totalUseRate += cmsInternalService.prometheus(expr, timeStamp);
                        break;
                    }
                }
            }

            double useRate = totalUseRate / (double) cloudServers.size();
            JSONObject jsonObject = JSON.parseObject(recordMetric.getScoreSetting());
            List<AssessConformityStandard.StandardInterval> standardIntervals = JSON.parseArray(jsonObject.getString("score"), AssessConformityStandard.StandardInterval.class);
            for (AssessConformityStandard.StandardInterval si : standardIntervals) {
                String condition = String.format("(%s%s%d && %s%s%d)", useRate, si.getOperator(), si.getValue(), useRate, si.getOperator2(), si.getValue2());
                log.info("===> 执行条件：{}", condition);
                if (executeCondition(condition)) {
                    recordMetric.setScore(Double.valueOf(si.getScore()));
                    recordMetric.setScoreLevel(si.getLevel());
                    recordMetric.setQualified(si.getQualified());
                }
            }
        }
    }

    /**
     * 执行条件
     *
     * @param condition 条件
     * @return 返回条件执行结果
     */
    private Boolean executeCondition(String condition) {
        try {
            return Boolean.parseBoolean(new ScriptEngineManager().getEngineByName("javascript").eval(condition).toString());
        } catch (ScriptException e) {
            log.error("执行条件{} 失败：{}", condition, e);
        }
        return false;
    }

    /**
     * 返回一个包含月中天数和时间戳的映射。
     *
     * @param cycleType  周期类型
     * @param cycleValue 周期值
     * @return 一个包含月中天数和时间戳的Map
     */
    private static Map<String, String> getDayAndTimestamp(AssessTask.CycleTypeEnum cycleType, String cycleValue) {
        switch (cycleType) {
            case YEAR:
                return getDayAndTimestampFromYear(cycleValue);
            case QUARTER:
                break;
            case MONTH:
                return getDayAndTimestampFromMonth(cycleValue);
            default:
                break;
        }
        return new HashMap<>(2);
    }

    private static Map<String, String> getDayAndTimestampFromYear(String cycleValue) {
        Calendar calendar = Calendar.getInstance();
        calendar.setLenient(false);
        calendar.clear();
        calendar.set(Integer.parseInt(cycleValue), Calendar.DECEMBER, 31);

        YearMonth ym = YearMonth.of(Integer.parseInt(cycleValue), 12);
        LocalDateTime endOfMonth = ym.atEndOfMonth().atTime(23, 59, 59);
        String timestamp = String.valueOf(endOfMonth.toEpochSecond(ZoneOffset.UTC));

        Map<String, String> map = new HashMap<>(2);
        map.put("timestamp", timestamp);
        map.put("dayOfMonth", String.valueOf(calendar.getActualMaximum(Calendar.DAY_OF_YEAR)));
        return map;
    }

    private static Map<String, String> getDayAndTimestampFromMonth(String cycleValue) {
        int year = Integer.parseInt(cycleValue.substring(0, 4));
        int month = Integer.parseInt(cycleValue.substring(4));

        YearMonth ym = YearMonth.of(year, month);
        int dayOfMonth = ym.lengthOfMonth();
        LocalDateTime endOfMonth = ym.atEndOfMonth().atTime(23, 59, 59);
        String timestamp = String.valueOf(endOfMonth.toEpochSecond(ZoneOffset.UTC));
        Map<String, String> map = new HashMap<>(2);
        map.put("timestamp", timestamp);
        map.put("dayOfMonth", String.valueOf(dayOfMonth));
        return map;
    }

    /**
     * 复杂头写入
     * 合并单元格
     *
     * @param recordId 考核记录ID
     * @param response 返回响应
     * @return void
     */
    public GeneralResult<Void> download(Long recordId, HttpServletResponse response) {
        ExcelWriter excelWriter = null;
        InputStream inputStream = null;
        ServletOutputStream outputStream = null;
        try {
            AssessRecord assessRecord = assessRecordRepository.getById(recordId);
            if (null == assessRecord) {
                return new GeneralResult<>(false, "没有对应的历史记录信息");
            }
            ClassPathResource classPathResource = new ClassPathResource("/template/template.xlsx");
            inputStream = classPathResource.getInputStream();
            outputStream = response.getOutputStream();
            List<AssessRecordMetric> recordMetrics = recordMetricRepository.queryByRecordId(assessRecord.getId());
            List<Long> ruleIds = recordMetrics.stream().map(AssessRecordMetric::getMetricRuleId).collect(Collectors.toList());
            List<AssessMetric> assessMetrics = metricRepository.queryByTaskId(assessRecord.getTaskId());
            List<AssessRule> assessRules = ruleRepository.queryByIds(ruleIds);
            JSONObject jsonObject = new JSONObject();
            excelWriter = EasyExcel.write(outputStream).inMemory(Boolean.TRUE).withTemplate(inputStream).excelType(ExcelTypeEnum.XLSX).build();
            WriteSheet writeSheet = EasyExcel.writerSheet().registerWriteHandler(new CustomMergeStrategy(data(recordMetrics, assessMetrics, assessRules, jsonObject, false).stream().map(item -> item.getString("metricName")).collect(Collectors.toList()), 0)).build();
            FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();
            excelWriter.fill(data(recordMetrics, assessMetrics, assessRules, jsonObject, true), fillConfig, writeSheet);
            Map<String, Object> map = MapUtils.newHashMap();
            map.put("name", assessRecord.getCycleValue() + assessRecord.getAssessObj() + "考核表");
            map.put("assessObj", assessRecord.getAssessObj());
            map.put("performanceLevel", assessRecord.getPerformanceLevel());
            map.put("scoreSettingTotal", jsonObject.getInteger("scoreSettingTotal"));
            map.put("scoreTotal", jsonObject.getInteger("scoreTotal"));
            map.put("executeScore", assessRecord.getExecuteScore());
            excelWriter.fill(map, writeSheet);
            String fileName = URLEncoder.encode(map.get("name").toString(), StandardCharsets.UTF_8).replaceAll("\\+", "%20");
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            excelWriter.finish();
            return null;
        } catch (Exception e) {
            log.error("download is fail:", e);
            return new GeneralResult<>(false, "下载失败");
        } finally {
            // 千万别忘记finish 会帮忙关闭流
            if (excelWriter != null) {
                excelWriter.finish();
            }
            //关闭流
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    log.error("download is fail:", e);
                }
            }
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    log.error("download is fail:", e);
                }
            }
        }
    }

    private List<JSONObject> data(List<AssessRecordMetric> recordMetrics, List<AssessMetric> assessMetrics, List<AssessRule> assessRules, JSONObject tempJsonObject, Boolean flag) {
        List<JSONObject> list = ListUtils.newArrayList();
        AtomicInteger scoreSettingTotalTemp = new AtomicInteger();
        AtomicDouble scoreTotalTemp = new AtomicDouble();
        recordMetrics.forEach(item -> {
            JSONObject data = new JSONObject();
            assessMetrics.stream().filter(metric -> metric.getId().equals(item.getMetricId())).findFirst().ifPresent(metric -> data.put("metricName", metric.getName()));
            assessRules.stream().filter(rule -> rule.getId().equals(item.getMetricRuleId())).findFirst().ifPresent(rule -> data.put("ruleName", rule.getRuleContent()));
            //找到规则中分数最高的数据作为分值
            AssessRule assessRule = assessRules.stream().filter(rule -> rule.getId().equals(item.getMetricRuleId())).findFirst().orElse(null);
            int maxScore = 0;
            if (null != assessRule) {
                JSONObject jsonObject = JSONObject.parseObject(assessRule.getScoreSetting(), JSONObject.class);
                JSONArray scoreArray = jsonObject.getJSONArray("score");
                for (int i = 0; i < scoreArray.size(); i++) {
                    JSONObject scoreObject = scoreArray.getJSONObject(i);
                    int score = scoreObject.getIntValue("score");
                    if (score > maxScore) {
                        maxScore = score;
                    }
                }
            }
            data.put("scoreSetting", maxScore);
            data.put("score", item.getScore());
            if (flag) {
                scoreSettingTotalTemp.addAndGet(maxScore);
                scoreTotalTemp.addAndGet(item.getScore());
            }
            list.add(data);
        });
        if (flag) {
            tempJsonObject.put("scoreSettingTotal", scoreSettingTotalTemp.get());
            tempJsonObject.put("scoreTotal", scoreTotalTemp.get());
        }
        return list;
    }

    public Object scoreRank(Long classifyId, String cycleValue, Integer num) {
        Map<String, Object> result = new HashMap<>(3);
        result.put("scoreRankList", scoreRanking(classifyId, cycleValue, null, null));
        result.put("metricPassRateList", metricPassRate(classifyId, cycleValue));
        result.put("scoreTrendList", scoreTrend(classifyId, num, cycleValue));
        return result;
    }

    /**
     * 指标合格率占比
     *
     * @param classifyId 分类ID
     * @param cycleValue 周期值（年月）
     * @return 指标合格率列表
     */
    public List<MetricPassRate> metricPassRate(Long classifyId, String cycleValue) {
        List<AssessRecord> assessRecords = assessRecordRepository.queryByClassifyIdAndCycleValue(classifyId, cycleValue);
        if (assessRecords.isEmpty()) {
            return Collections.emptyList();
        }

        List<Long> recordIds = assessRecords.stream().map(AssessRecord::getId).collect(Collectors.toList());

        List<AssessRecordMetric> assessRecordMetrics = recordMetricRepository.queryByRecordIds(recordIds);

        List<MetricPassRate> metricPassRateList = new ArrayList<>();

        for (AssessRecord assessRecord : assessRecords) {
            MetricPassRate metricPassRate = new MetricPassRate();
            metricPassRate.setName(assessRecord.getAssessObj());

            long total = assessRecordMetrics.stream().filter(assessRecordMetric -> assessRecordMetric.getRecordId().equals(assessRecord.getId())).count();

            metricPassRate.setTotal(total);
            metricPassRate.setCountQualified(assessRecord.getQualifiedCount());
            metricPassRate.setConutUnQualified(assessRecord.getUnqualifiedCount());

            BigDecimal qualifiedPercent = BigDecimal.valueOf(assessRecord.getQualifiedCount()).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(total), 5, RoundingMode.HALF_UP);

            BigDecimal unQualifiedPercent = BigDecimal.valueOf(assessRecord.getUnqualifiedCount()).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(total), 5, RoundingMode.HALF_UP);

            metricPassRate.setQualifiedPercent(qualifiedPercent.doubleValue() + "%");
            metricPassRate.setUnQualifiedPercent(unQualifiedPercent.doubleValue() + "%");

            metricPassRateList.add(metricPassRate);
        }

        return metricPassRateList;
    }

    /**
     * 考核分数趋势统计
     *
     * @param classifyId 分类ID
     * @param year       年份
     * @return 返回考核分数趋势
     */
    public Object scoreTrend(Long classifyId, String year) {
        return scoreTrend(classifyId, 12, MessageFormat.format("{0}12", year));
    }

    /**
     * 云服务商绩效分值趋势
     *
     * @param classifyId 分类ID
     * @param num        数量
     * @param cycleValue 周期值（年月）
     * @return 返回云服务商绩效分值趋势数据
     */
    public Object scoreTrend(Long classifyId, Integer num, String cycleValue) {
        List<String> previousMonths = DateUtil.getPreviousMonths(num, cycleValue).reversed();
        List<AssessRecord> assessRecords = assessRecordRepository.scoreTrend(classifyId, previousMonths);
        if (assessRecords.isEmpty()) {
            return null;
        }

        List<RequestData> values = new ArrayList<>();
        Set<String> objects = assessRecords.stream().map(AssessRecord::getAssessObj).collect(Collectors.toSet());

        for (String obj : objects) {
            RequestData data = new RequestData();
            data.setName(obj);

            List<Double> scores = new ArrayList<>();

            for (String month : previousMonths) {
                boolean isFound = false;
                for (AssessRecord record : assessRecords) {
                    if (month.equals(record.getCycleValue()) && obj.equals(record.getAssessObj())) {
                        scores.add(null == record.getExecuteScore() ? 0.0 : record.getExecuteScore());
                        isFound = true;
                        break;
                    }
                }
                if (!isFound) {
                    scores.add(0.0);
                }
            }
            data.setData(scores);
            values.add(data);
        }

        Map<String, Object> result = new HashMap<>(2);
        result.put("keys", previousMonths);
        result.put("values", values);
        return result;
    }

    public Object scoreRange(Long classifyId, String cycleValue, String sort, Integer limit) {
        return scoreRanking(classifyId, cycleValue, sort, limit);
    }

    /**
     * 绩效分值排名
     *
     * @param cycleValue 周期值（年月）
     * @return
     */
    public Object scoreRanking(Long classifyId, String cycleValue, String sort, Integer limit) {
        List<AssessRecord> assessRecords = assessRecordRepository.scoreRanking(classifyId, cycleValue, sort, limit);
        if (assessRecords.isEmpty()) {
            return null;
        }
        List<String> keys = assessRecords.stream().map(AssessRecord::getAssessObj).collect(Collectors.toList());

        List<Double> scores = assessRecords.stream().map(AssessRecord::getExecuteScore).collect(Collectors.toList());

        Map<String, Object> result = new HashMap<>(2);
        result.put("keys", keys);

        List<RequestData> values = new ArrayList<>();
        RequestData data = new RequestData();
        data.setName("分数");
        data.setData(scores);
        values.add(data);
        result.put("values", values);

        return result;
    }

    public Object scoreProportion(Long classifyId, String cycleValue) {
        AssessScoreProportionModel result = new AssessScoreProportionModel();
        List<AssessRecord> assessRecords = assessRecordRepository.queryByClassifyIdAndCycleValue(classifyId, cycleValue);
        if (assessRecords.isEmpty()) {
            return result;
        }
        result.setTotal(assessRecords.size());

        int ninetyToHundredCount = 0;
        int sixtyToNinetyCount = 0;
        int zeroToSixtyCount = 0;

        for (AssessRecord assessRecord : assessRecords) {
            double executeScore = assessRecord.getExecuteScore();
            if (executeScore >= 90 && executeScore <= 100) {
                ninetyToHundredCount++;
            } else if (executeScore >= 60 && executeScore < 90) {
                sixtyToNinetyCount++;
            } else if (executeScore >= 0 && executeScore < 60) {
                zeroToSixtyCount++;
            }
        }

        result.setNinetyToHundredCount(ninetyToHundredCount);
        result.setNinetyToHundredPercent(ninetyToHundredCount / (double) assessRecords.size() * 100 + "%");
        result.setSixtyToNinetyCount(sixtyToNinetyCount);
        result.setSixtyToNinetyPercent(sixtyToNinetyCount / (double) assessRecords.size() * 100 + "%");
        result.setZeroToSixtyCount(zeroToSixtyCount);
        result.setZeroToSixtyPercent(zeroToSixtyCount / (double) assessRecords.size() * 100 + "%");

        return result;
    }

    public GridBean<AssessRecord> scoreResult(Pager pager, Long classifyId, String cycleValue) {
        pager.setSorter(pager.getSorter() != null ? pager.getSorter() : new HashMap<>(2));
        pager.setParams(pager.getParams() != null ? pager.getParams() : new ArrayList<>());
        pager.getSorter().put("gmtCreate", Common.ONE);
        int total = assessRecordRepository.countByClassifyIdAndCycleValue(classifyId, cycleValue);
        List<AssessRecord> list = assessRecordRepository.queryByClassifyIdAndCycleValue(pager.getPage(), pager.getRows(), pager.getSorter(), classifyId, cycleValue);
        return new GridBean<>(pager.getPage(), pager.getRows(), total, list);
    }

    public Object scoreResultDetail(Long recordId) {
        return recordMetricRepository.detailOfTask(recordId);
    }

    @Data
    static class RequestData {
        String name;
        List<Double> data;
    }

    @Data
    static class MetricPassRate {
        String name;
        Long total;
        Integer countQualified;
        Integer conutUnQualified;
        String qualifiedPercent;
        String unQualifiedPercent;
    }
}
