package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.megatron.common.model.GeneralResult;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.RequestContext;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 服务操作业务实现
 *
 * @author YuFengYang
 * @version 1.0
 * @since 2022/2/15 9:40
 */
@Slf4j
@Component
public class FmsFlowService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public FmsFlowService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }


    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create("cos");


    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    public GeneralResult startInstanceById(Long id, RequestContext context, Long project, Map<String, Object> args){
        HashMap<String, Object> map = new HashMap<>();
        map.put("defId",id);
        map.put("context",JSONObject.toJSONString(context));
        map.put("project", null == project ? -1 : project);
        map.put("params",JSONObject.toJSONString(args));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/fms/v1/instances", HttpMethod.POST, headers,
                map);
        GeneralResult invoke = remoteService.invoke();
        if (null == invoke || invoke.isFailed()){
            return new GeneralResult(false,"实例启动失败!");
        }
        return invoke;
    }

    public long countTodo(RequestContext context) {
        HashMap<String, Object> map = new HashMap<>();
        map.put("context",JSONObject.toJSONString(context));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/fms/v1/instances/count", HttpMethod.GET, headers,
                map);
        GeneralResult invoke = remoteService.invoke();
        if (invoke.isFailed()){
            log.error("获取待审批工单数量异常，异常原因{}",invoke.getData());
            return 0L;
        }
        return Long.parseLong(invoke.getData().toString());
    }
}
