package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.internel.*;
import com.google.common.collect.Lists;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * cmp内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月9日
 */
@Slf4j
@Component
public class ImsInternalService implements InitializingBean {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    private final Map<String, Object> headers = new HashMap<>(4);
    private static final Service SERVICE = Service.create("cmp");

    public GeneralResult recycle(String uuid) {
        Map<String, Object> paramMap = MapTools.simpleMap("action", "recycleByUuid");
        JSONObject params = new JSONObject();
        params.put("uuid", uuid);
        paramMap.put("params", params.toJSONString());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms/0", HttpMethod.PATCH, headers, paramMap);
        return remoteService.invoke();
    }

    public CloudServer queryBySn(String uuid) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("uuid", uuid), Sign.EQ));
        Pager pager = new Pager(1, 1, params, null);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", HttpMethod.GET, headers, pager.toMap());
        GeneralResult<?> result = remoteService.invoke();
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        if (list.isEmpty()) {
//            throw new RuntimeException("cloudServer not found, uuid:" + uuid);
            return null;
        }
        return list.get(0);
    }

    public List<CloudServer> queryByParams(List<Param> params) {
        Pager pager = new Pager(1, 999, params, null);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", HttpMethod.GET, headers, pager.toMap());
        GeneralResult<?> result = remoteService.invoke();
        if (result == null || result.isFailed()) {
            log.warn("未找到云主机，请求参数：" + JSON.toJSONString(params));
            return Lists.newArrayList();
        }
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    public List<Volume> listVms() {
        Pager pager = new Pager(1, 9999, new ArrayList<>(), null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/volumes", HttpMethod.GET, headers, requestMap);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            throw new RuntimeException(result.getMessage());
        }
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Volume.class);
    }

    public List<Volume> listByVmId(Long serverId, String vendorType) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listByVmId");
        object.put("serverId", serverId);
        String uri = String.format("/plugins/%s/v1/volumes/condition", vendorType.toLowerCase());
        ;
        RemoteService remoteService = serviceFactory.build(SERVICE, uri, HttpMethod.GET, headers, MapTools.simpleMap(Common.CONDITION, object));
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            throw new RuntimeException(result.getMessage());
        }
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Volume.class);
    }

    /**
     * 查询SmartX云硬盘列表
     *
     * @param serverId
     * @param vendorId
     * @return
     */
    public List<Volume> getVolumesForSmartX(Long serverId, Long vendorId, String vendorType) {
        List<Param> paramList = Lists.newArrayList(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        Pager pager = new Pager(1, Integer.MAX_VALUE, paramList, null);
        Map<String, Object> params = pager.toMap();
        params.put("action", "attached");
        String uri = String.format("/v1/vms/%d/volumes", serverId);
        if (StringUtils.isNotBlank(vendorType)) {
            uri = String.format("/plugins/%s%s", vendorType.toLowerCase(), uri);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, uri, HttpMethod.GET, headers, params);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            throw new RuntimeException(result.getMessage());
        }
        JSONArray rows = JSONObject.parseObject(JSONObject.toJSONString(result.getData())).getJSONArray("rows");
        return JSONArray.parseArray(rows.toJSONString(), Volume.class);
    }

    public PoolGroup getPoolGroup(Long poolGroupId) {
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/pool/groups/" + poolGroupId, HttpMethod.GET, headers, null);
        GeneralResult<?> invoke = remoteService.invoke();
        Assert.isTrue(invoke.isSuccess(), invoke.getMessage());
        return JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()), PoolGroup.class);
    }

    public GeneralResult serverDetail(Long serverId) {
        JSONObject object = new JSONObject();
        object.put("serverId", serverId);
        object.put(Common.CONDITION, "basic");
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms/condition", HttpMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    public GeneralResult delete(List<String> instanceIds) {
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("action", "removeByInstanceIds");
        paramMap.put(Common.PARAMS, JSONObject.toJSONString(instanceIds));
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", HttpMethod.PATCH, headers, paramMap);
        return remoteService.invoke();
    }

    /**
     * 获取云平台列表
     *
     * @return
     */
    public List<CloudVendor> getVendorList() {
        ArrayList<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("vendorCategory", "COMPUTE"), Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", HttpMethod.GET, headers, requestMap);
        GeneralResult invoke = remoteService.invoke();
        Assert.notNull(invoke.getData(), "获取平台列表失败");
        JSONObject vendorPager = JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()));
        JSONArray rows = vendorPager.getJSONArray("rows");
        return JSONArray.parseArray(rows.toJSONString(), CloudVendor.class);
    }

    /**
     * 获取云平台信息
     *
     * @return
     */
    public CloudVendor getVendorDetail(Long id) {
        ArrayList<Param> params = new ArrayList<>();
        Map<String, Object> paramsMap = MapTools.simpleMap("vendorCategory", "COMPUTE");
        paramsMap.put("id", id);
        params.add(new Param(paramsMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vendors", HttpMethod.GET, headers, requestMap);
        GeneralResult invoke = remoteService.invoke();
        Assert.notNull(invoke.getData(), "获取平台信息失败");
        JSONObject vendorPager = JSONObject.parseObject(JSONObject.toJSONString(invoke.getData()));
        JSONArray rows = vendorPager.getJSONArray("rows");
        List<CloudVendor> cloudVendors = JSONArray.parseArray(rows.toJSONString(), CloudVendor.class);
        Assert.isTrue(cloudVendors.size() <= 1, "more than one vendor in the list");
        return cloudVendors.isEmpty() ? null : cloudVendors.get(0);
    }

    /**
     * 获取主机列表
     *
     * @return
     */
    public GeneralResult getHostList(Long vendorId, Long vdcId, Long clusterId, String name) {
        return this.getHostList(vendorId, vdcId, clusterId, name, null);
    }

    /**
     * 获取主机列表
     *
     * @return
     */
    public GeneralResult getHostList(Long vendorId, Long vdcId, Long clusterId, String name, String vendorType) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByCondition");
        jsonObject.put("vendorId", vendorId);
        jsonObject.put("vendorType", vendorType);
        jsonObject.put("vdcId", vdcId);
        jsonObject.put("clusterId", clusterId);
        jsonObject.put("name", name);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/hosts/condition", HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    public List<Server> listHosts() {
        List<Server> listHosts = JSON.parseArray(this.redisTemplate.opsForValue().get("listHosts"), Server.class);
        if (ListTool.isEmpty(listHosts)) {
            List<Param> params = new ArrayList<>();
            Pager pager = new Pager(1, 9999, params, null);
            Map<String, Object> requestMap = pager.toMap();
            RemoteService remoteService =
                    serviceFactory.build(SERVICE, "/v1/hosts", HttpMethod.GET, headers, requestMap);
            GeneralResult result = remoteService.invoke();
            if (result.isFailed()) {
                return new ArrayList<>();
            }
            String content = JSONObject.toJSONString(result.getData());
            if (StringUtils.isEmpty(content)) {
                return new ArrayList<>();
            }
            GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
            listHosts = JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), Server.class);
            this.redisTemplate.opsForValue().set("listHosts", JSON.toJSONString(listHosts), 15, TimeUnit.MINUTES);
        }
        return listHosts;
    }

    /**
     * 获取云主机列表
     *
     * @return
     */
    public GeneralResult getVmList(Long vendorId, Long vdcId, Long clusterId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listByCondition");
        jsonObject.put("vendorId", vendorId);
        jsonObject.put("vdcId", vdcId);
        jsonObject.put("clusterId", clusterId);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取云硬盘
     *
     * @param vendorId
     * @return
     */
    public GeneralResult getVolume(Long vendorId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("condition", "listByCondition");
        jsonObject.put("vendorId", vendorId);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/volumes/condition", HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取数据存储
     *
     * @param vendorId
     * @return
     */
    public GeneralResult getDataStore(Long vendorId, Long vdcId) {
        ArrayList<Param> params = new ArrayList<>();
        if (null != vendorId && 0L != vendorId) {
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        if (null != vdcId && 0L != vdcId) {
            params.add(new Param(MapTools.simpleMap("vdcId", vdcId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/datastores", HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    /**
     * 获取浮动Ip
     *
     * @param vendorId
     * @return
     */
    public GeneralResult getFloatIp(Long vendorId) {
        ArrayList<Param> params = new ArrayList<>();
        if (null != vendorId && 0L != vendorId) {
            params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/floatips", HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    public List<CloudServer> listServers(RequestContext context) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("isRecycle", 1), Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vms", HttpMethod.GET, headers, requestMap);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(content)) {
            return new ArrayList<>();
        }
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudServer.class);
    }

    /**
     * 获取已使用Ip
     *
     * @param poolName
     * @return
     */
    public GeneralResult getIpNumber(String poolName) {
        ArrayList<Param> params = new ArrayList<>();
        if (poolName != null) {
            params.add(new Param(MapTools.simpleMap("poolName", poolName), Sign.LK));
        }
        Pager pager = new Pager(1, 9999, params, null);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ips", HttpMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }

    /**
     * 获取ip池列表
     *
     * @param poolName
     * @return
     */
    public GeneralResult getIpPoolList(String poolName) {
        ArrayList<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("countIp", true), Sign.EQ));
        if (null != poolName) {
            params.add(new Param(MapTools.simpleMap("name", poolName), Sign.LK));
        }
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/ip/pools", HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    public List<CloudServer> listVmsByVendor(String vendorType, Boolean isAll, RequestContext context) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put(Common.CONDITION, "listTenantProject");
        jsonObject.put("isAll", isAll);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        String uri = "/v1/vms/condition";
        RemoteService remoteService = serviceFactory.build(SERVICE, uri, HttpMethod.GET, headers, requestMap);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed() || null == result.getData()) {
            return new ArrayList<>();
        }
        List<CloudServer> vms = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudServer.class);
        this.redisTemplate.opsForValue().set("listVmsByVendor", JSON.toJSONString(vms), 120, TimeUnit.SECONDS);
        if (StringUtils.isNotBlank(vendorType)) {
            vms = vms.stream().filter(vm -> vendorType.equals(vm.getVendorType())).collect(Collectors.toList());
        }
        return vms;
    }

    /**
     * 获取指定类型云主机列表
     *
     * @param context
     * @return
     */
    public List<CloudServer> listVmsByDcId(Long dcId, Boolean isAll, RequestContext context) {
        JSONObject jsonObject = new JSONObject();
        List<Long> dcVendorIds = null;
        if (null != dcId && 0L != dcId) {
            List<CloudVendor> cloudVendors = this.listCloudVendor(null);
            dcVendorIds = cloudVendors.stream()
                    .filter(cloudVendor -> dcId.equals(cloudVendor.getDcId())).map(CloudVendor::getId)
                    .collect(Collectors.toList());
        }
        jsonObject.put(Common.CONDITION, "listTenantProject");
        jsonObject.put("isAll", isAll);
        Map<String, Object> requestMap = MapTools.simpleMap(Common.CONDITION, jsonObject);
        RemoteService remoteService =
                serviceFactory.build(SERVICE, "/v1/vms/condition", HttpMethod.GET, headers, requestMap);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed() || null == result.getData()) {
            return new ArrayList<>();
        }
        List<CloudServer> cloudServers = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), CloudServer.class);
        List<Long> finalDcVendorIds = dcVendorIds;
        return cloudServers.stream()
                .filter(cloudServer -> null == finalDcVendorIds || finalDcVendorIds.contains(cloudServer.getVendorId()))
                .collect(Collectors.toList());
    }

    public List<CloudVendor> listCloudVendor(String type) {
        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("vendorCategory", "COMPUTE");
        if (StringUtils.isNotBlank(type)) {
            paramMap.put("type", type);
        }
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        Map<String, Object> headers = new HashMap<>();
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/vendors", HttpMethod.GET, headers, requestMap);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            log.error(result.getMessage());
            return new ArrayList<>();
        }
        String content = JSONObject.toJSONString(result.getData());
        GridBean gridBean = JSONObject.parseObject(content, GridBean.class);
        List<CloudVendor> cloudVendors = JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), CloudVendor.class);
        this.redisTemplate.opsForValue().set("listCloudVendor", JSON.toJSONString(cloudVendors), 120, TimeUnit.SECONDS);
        return cloudVendors;
    }

    public List<NetworkCard> listNetworkCards(Long templateId, String vendorType) {
        List<Param> params = new ArrayList<>();
        Map<String, Object> paramMap = MapTools.simpleMap("hostId", templateId);
        paramMap.put("category", "VM");
        params.add(new Param(paramMap, Sign.EQ));
        Pager pager = new Pager(1, 9999, params, null);
        Map<String, Object> requestMap = pager.toMap();
        String uri = String.format("/plugins/%s/v1/nics", vendorType.toLowerCase());
        RemoteService remoteService = serviceFactory.build(SERVICE, uri, HttpMethod.GET, headers, requestMap);
        GeneralResult<?> result = remoteService.invoke();
        Assert.isTrue(result.isSuccess(), result.getMessage());
        String content = JSONObject.toJSONString(result.getData());
        GridBean<?> gridBean = JSONObject.parseObject(content, GridBean.class);
        return JSONArray.parseArray(JSONObject.toJSONString(gridBean.getRows()), NetworkCard.class);
    }

    public GeneralResult<?> selectAllByVendorId(Long vendorId, String vendorType) {
        Map<String, Object> requestMap = MapTools.simpleMap("vendorId", vendorId);
        String uri = String.format("/plugins/%s/v1/vms/selectAllByVendorId", vendorType.toLowerCase());
        RemoteService remoteService = serviceFactory.build(SERVICE, uri, HttpMethod.GET, headers, requestMap);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
    }

}
