package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.internel.Department;
import com.bocloud.cos.entity.internel.Dictionary;
import com.bocloud.cos.entity.internel.Permission;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 服务授权
 *
 * @author Administrator
 */
@Component
@Slf4j
public class SmsInternalService implements InitializingBean {

    @Autowired
    private ServiceFactory serviceFactory;
    @Autowired
    private CurrentService currentService;
    private Map<String, Object> headers = new HashMap<>(4);
    private static Service service = Service.create("sms");

    public String getSystemConfig(String code) {
        Map<String, Object> paramMap = MapTools.simpleMap("code", code);
        RemoteService remote = serviceFactory.build(service, "/v1/sysconfigs", HttpMethod.GET, headers, paramMap);
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        Object data = result.getData();
        if (null == data) {
            return null;
        }
        return data.toString();
    }

    public String getToken(Long targetId, String catalog) {
        Map<String, Object> paramMap = MapTools.simpleMap("targetId", targetId);
        paramMap.put("catalog", catalog);
        RemoteService remoteService = serviceFactory.build(service, "/v1/sso/certification",
                HttpMethod.GET, headers, paramMap);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return String.valueOf(result.getData());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    public List<Permission> getPermission(String name) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listByName");
        object.put("name", name);
        RemoteService remoteService = serviceFactory.build(service, "/v1/permissions/condition", HttpMethod.GET, headers, MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Permission.class);
    }

    public List<Department> listDepart() {
        RemoteService remoteService = serviceFactory.build(service, "/v1/departments", HttpMethod.GET, headers, MapTools.simpleMap("parentId", 0));
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Department.class);
    }

    public Map<Long, String> getDepartMap(Long parentId) {
        if(null == parentId){
            parentId = 0L;
        }
        RemoteService remoteService = serviceFactory.build(service, "/v1/departments", HttpMethod.GET, headers, MapTools.simpleMap("parentId", parentId));
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        List<Department> list = JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Department.class);
        Map<Long, String> map = new HashMap<>();
        return getDepartMap(list, map);
    }

    private Map<Long, String> getDepartMap(List<Department> list, Map<Long, String> map) {
        for (Department depart : list) {
            map.put(depart.getId(), depart.getName());
            if (!CollectionUtils.isEmpty(depart.getChildren())) {
                getDepartMap(depart.getChildren(), map);
            }
        }
        return map;
    }

    public Department getDepart(Long departId) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/departments/" + departId, HttpMethod.GET, headers, new HashMap<>());
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return null;
        }
        return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Department.class);
    }

    public GeneralResult updatePermission(Permission permission) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/permissions/" + permission.getId(), HttpMethod.PUT, headers, permission);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    public Boolean getLicenseService(String serviceCode) {
        JSONObject object = new JSONObject();
        RemoteService remoteService = serviceFactory.build(service, "/v2/plugins/vendors", HttpMethod.GET, headers, object);
        GeneralResult<List<JSONObject>> invoke = (GeneralResult<List<JSONObject>>) remoteService.invoke();
        List<JSONObject> data = invoke.getData();
        return isInclude(data, serviceCode);
    }

    public Boolean isInclude(List<JSONObject> dictionaryList, String code) {
        for (JSONObject o : dictionaryList) {
            List<JSONObject> collect = o.getJSONArray("children").toJavaList(JSONObject.class).stream()
                    .filter(i -> (null != i.getString("value") && code.equals(i.getString("value"))))
                    .collect(Collectors.toList());
            if (collect.size() > 0) {
                log.info("查询到绑定服务: {}", JSONObject.toJSONString(collect));
                return true;
            }
        }
        return false;
    }

    public Map<String, String> dictChildrenList(String code) {
        RemoteService remoteService = serviceFactory.build(service, "/v1/dictionaries/children", HttpMethod.GET, headers,
                MapTools.simpleMap("value", code));
        GeneralResult invoke = remoteService.invoke();
        Map<String, String> result = new HashMap<>();
        if (invoke.getData() != null) {
            List<Dictionary> dictionaries =
                    JSONArray.parseArray(JSONObject.toJSONString(invoke.getData()), Dictionary.class);
            for (Dictionary dictionary : dictionaries) {
                result.put(dictionary.getValue(), dictionary.getName());
            }
        }
        return result;
    }


}
