package com.bocloud.cos.service.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cos.entity.internel.Project;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Component
public class SmsProjectService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsProjectService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    private static final Service SERVICE = Service.create("sms");

    public List<Project> listTenantProject(Long tenantId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listTenantProject");
        if (null != tenantId && 0 != tenantId) {
            object.put("tenantId", tenantId);
        }
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/projects", HttpMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return Lists.newArrayList();
        }
        String data = JSONObject.toJSONString(result.getData());
        if (!StringUtils.isNotEmpty(data)) {
            return Lists.newArrayList();
        }
        return JSONArray.parseArray(data, Project.class);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

    /**
     * 查询租户下项目
     *
     * @param tenantId
     * @return
     */
    public List<Project> listTenantProjects(Long tenantId) {
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/tenants/" + tenantId + "/projects", HttpMethod.GET,
                headers, null);
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return Lists.newArrayList();
        }
        String data = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(data)) {
            return Lists.newArrayList();
        }
        return JSONArray.parseArray(data, Project.class);
    }

    /**
     * 查询用户管理的项目
     *
     * @param userId
     * @return
     */
    public List<Project> listUserManagerProjects(Long userId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listUserManagerProjects");
        object.put("userId", userId);
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/projects/condition", HttpMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return Lists.newArrayList();
        }
        String data = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(data)) {
            return Lists.newArrayList();
        }
        return JSONArray.parseArray(data, Project.class);
    }

    /**
     * 查询用户有可见权限的项目
     *
     * @param userId
     * @return
     */
    public List<Project> listUserVisibleProjects(Long userId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "listUserVisibleProjects");
        object.put("userId", userId);
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/projects/condition", HttpMethod.GET, headers,
                MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return Lists.newArrayList();
        }
        String data = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(data)) {
            return Lists.newArrayList();
        }
        return JSONArray.parseArray(data, Project.class);
    }

    public List<Project> listByDepart(String departName) {
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/projects/by/depart", HttpMethod.GET, headers,
                MapTools.simpleMap("departName", departName));
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return Lists.newArrayList();
        }
        String data = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(data)) {
            return Lists.newArrayList();
        }
        return JSONArray.parseArray(data, Project.class);
    }

    public Project listByProjectName(String projectName) {
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/projects/by/name", HttpMethod.GET, headers,
                MapTools.simpleMap("projectName", projectName));
        GeneralResult result = remote.invoke();
        if (result.isFailed()) {
            return null;
        }
        String data = JSONObject.toJSONString(result.getData());
        if (StringUtils.isEmpty(data)) {
            return null;
        }
        return JSONObject.parseObject(data, Project.class);
    }
}
