package com.bocloud.cos.utils;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.usermodel.charts.ChartLegend;
import org.apache.poi.ss.usermodel.charts.LegendPosition;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFChart;
import org.apache.poi.xssf.usermodel.XSSFDrawing;
import org.openxmlformats.schemas.drawingml.x2006.chart.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Created by liuyuanyuan on 2020/7/16 15:45
 */
@Slf4j
public class ExcelChartUtil {

    private static SXSSFWorkbook workbook;

    public static void createBarChart(SXSSFSheet hiddenSheet, List<String> fldNameArr, List<Map<String, Object>> dataList, String titleName, String ip, Boolean isDlbls) throws Exception {
        // 创建一个sheet页
        SXSSFSheet barSheet = workbook.createSheet("柱状图图表页");
        // drawSheet0Table(sheet,titleArr,fldNameArr,dataList);
        // 堆积=STBarGrouping.STACKED 多组=STBarGrouping.CLUSTERED
        drawSheetMap(hiddenSheet, barSheet, STBarGrouping.CLUSTERED, fldNameArr, dataList, titleName, ip, isDlbls);
    }

    /**
     * 创建折线图
     *
     * @param fldNameArr
     * @param dataList
     */
    public static void createTimeXYChar(SXSSFSheet hiddenSheet, List<String> fldNameArr, List<Map<String, Object>> dataList, String titleName, String ip) {
        // 创建一个sheet页
        SXSSFSheet lineSheet = workbook.createSheet("折线图图表页");
        // 第二个参数折线图类型:line=普通折线图,line-bar=折线+柱状图
        drawLineSheetMap(hiddenSheet, lineSheet, "line", fldNameArr, dataList, titleName, ip, 1);
    }

    /**
     * 生成柱状图
     *
     * @param sheet
     * @param group
     * @param fldNameArr
     * @param dataList
     * @return
     */
    private static void drawSheetMap(SXSSFSheet hiddenSheet, SXSSFSheet sheet, STBarGrouping.Enum group, List<String> fldNameArr,
                                     List<Map<String, Object>> dataList, String titleName, String ip, Boolean isDlbls) {
        // 获取隐藏sheet名称
        String sheetName = hiddenSheet.getSheetName();
        //写入标题
        drawTitle(sheet, titleName, dataList, ip);
        // 创建一个画布
        Drawing<?> drawing = sheet.createDrawingPatriarch();
        // 画一个图区域
        // 前四个默认0，从第8行到第25行,从第0列到第6列的区域
        ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 0, 4, 18, 30);
        // 创建一个chart对象
        XSSFChart chart = ((XSSFDrawing) drawing).createChart(anchor);
        // 获取CTChart对象
        CTChart ctChart = chart.getCTChart();
        CTPlotArea ctPlotArea = ctChart.getPlotArea();
        // 创建柱状图模型
        CTBarChart ctBarChart = ctPlotArea.addNewBarChart();
        CTBoolean ctBoolean = ctBarChart.addNewVaryColors();
        ctBarChart.getVaryColors().setVal(true);
        // 设置图类型
        ctBarChart.addNewGrouping().setVal(group);
        ctBoolean.setVal(true);
        ctBarChart.addNewBarDir().setVal(STBarDir.COL);
        // 是否添加左侧坐标轴
        ctChart.addNewDispBlanksAs().setVal(STDispBlanksAs.ZERO);
        ctChart.addNewShowDLblsOverMax().setVal(true);
        // 设置这两个参数是为了在STACKED模式下生成堆积模式；(standard)标准模式时需要将这两行去掉
        if ("stacked".equals(group.toString()) || "percentStacked".equals(group.toString())) {
            ctBarChart.addNewGapWidth().setVal(150);
            ctBarChart.addNewOverlap().setVal((byte) 100);
        }
        // 创建序列,并且设置选中区域
        for (int i = 0; i < fldNameArr.size(); i++) {
            CTBarSer ctBarSer = ctBarChart.addNewSer();
            CTSerTx ctSerTx = ctBarSer.addNewTx();
            // 图例区
            CTStrRef ctStrRef = ctSerTx.addNewStrRef();
            // 选定区域第0行,第1,2,3列标题作为图例 //1 2 3
            String legendDataRange = new CellRangeAddress(0, 0, i + 1, i + 1).formatAsString(sheetName, true);
            ctStrRef.setF(legendDataRange);
            ctBarSer.addNewIdx().setVal(i);
            // 横坐标区
            CTAxDataSource cttAxDataSource = ctBarSer.addNewCat();
            ctStrRef = cttAxDataSource.addNewStrRef();
            // 选第0列,第1-6行作为横坐标区域
            String axisDataRange = new CellRangeAddress(1, dataList.size(), 0, 0).formatAsString(sheetName, true);
            ctStrRef.setF(axisDataRange);
            // 数据区域
            CTNumDataSource ctNumDataSource = ctBarSer.addNewVal();
            CTNumRef ctNumRef = ctNumDataSource.addNewNumRef();
            // 选第1-6行,第1-3列作为数据区域 //1 2 3
            String numDataRange = new CellRangeAddress(1, dataList.size(), i + 1, i + 1).formatAsString(sheetName,
                    true);
            System.out.println(numDataRange);
            ctNumRef.setF(numDataRange);
            // 添加柱状边框线
            ctBarSer.addNewSpPr().addNewLn().addNewSolidFill().addNewSrgbClr().setVal(new byte[]{0, 0, 0});
            // 设置负轴颜色不是白色
            ctBarSer.addNewInvertIfNegative().setVal(false);
            // 设置标签格式
            ctBoolean.setVal(false);
            if (isDlbls) {
                CTDLbls newDLbls = ctBarSer.addNewDLbls();
                newDLbls.setShowLegendKey(ctBoolean);
                ctBoolean.setVal(true);
                newDLbls.setShowVal(ctBoolean);
                ctBoolean.setVal(false);
                newDLbls.setShowCatName(ctBoolean);
                newDLbls.setShowSerName(ctBoolean);
                newDLbls.setShowPercent(ctBoolean);
                newDLbls.setShowBubbleSize(ctBoolean);
                newDLbls.setShowLeaderLines(ctBoolean);
            }
        }
        // 告诉BarChart它有坐标轴，并给它们id
        ctBarChart.addNewAxId().setVal(123456);
        ctBarChart.addNewAxId().setVal(123457);
        // 横坐标
        CTCatAx ctCatAx = ctPlotArea.addNewCatAx();
        ctCatAx.addNewAxId().setVal(123456); // id of the cat axis
        CTScaling ctScaling = ctCatAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        ctCatAx.addNewAxPos().setVal(STAxPos.B);
        ctCatAx.addNewCrossAx().setVal(123457); // id of the val axis
        ctCatAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // 纵坐标
        CTValAx ctValAx = ctPlotArea.addNewValAx();
        ctValAx.addNewAxId().setVal(123457); // id of the val axis
        ctScaling = ctValAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        // 设置位置
        ctValAx.addNewAxPos().setVal(STAxPos.L);
        ctValAx.addNewCrossAx().setVal(123456); // id of the cat axis
        ctValAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // 是否删除主左边轴
        ctValAx.addNewDelete().setVal(false);
        // 是否删除横坐标
        ctCatAx.addNewDelete().setVal(false);
        // legend图注
        // if(true){
        CTLegend ctLegend = ctChart.addNewLegend();
        ctLegend.addNewLegendPos().setVal(STLegendPos.B);
        ctLegend.addNewOverlay().setVal(false);
        // }
    }

    /**
     * 生成折线图
     *
     * @param sheet
     * @param type
     * @param fldNameArr
     * @param dataList
     * @return
     */
    private static void drawLineSheetMap(SXSSFSheet hiddenSheet, SXSSFSheet sheet, String type, List<String> fldNameArr,
                                         List<Map<String, Object>> dataList, String titleName, String ip, int num) {
        // 获取隐藏页sheet名称
        String sheetName = hiddenSheet.getSheetName();
        //写入标题
        if (1 == num) {
            drawTitle(sheet, titleName, dataList, ip);
        }
        // 创建一个画布
        Drawing<?> drawing = sheet.createDrawingPatriarch();
        // 画一个图区域
        // 前四个默认0，从第8行到第25行,从第0列到第6列的区域1
        ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 0, 4 + 30 * (num - 1), 18, 30 * num);
        // 创建一个chart对象
        XSSFChart chart = ((XSSFDrawing) drawing).createChart(anchor);
        // 获取CTChart对象
        CTChart ctChart = chart.getCTChart();
        CTPlotArea ctPlotArea = ctChart.getPlotArea();
        // 折线图
        CTLineChart ctLineChart = ctPlotArea.addNewLineChart();
        CTBoolean ctBoolean = ctLineChart.addNewVaryColors();
        ctLineChart.addNewGrouping().setVal(STGrouping.STANDARD);
        // 创建序列,并且设置选中区域
        for (int i = 0; i < fldNameArr.size(); i++) {
            CTLineSer ctLineSer = ctLineChart.addNewSer();
            CTSerTx ctSerTx = ctLineSer.addNewTx();
            // 图例区
            CTStrRef ctStrRef = ctSerTx.addNewStrRef();
            // 选定区域第0行,第1,2,3列标题作为图例 //1 2 3
            String legendDataRange = new CellRangeAddress(0, 0, i + 1, i + 1).formatAsString(sheetName, true);
            ctStrRef.setF(legendDataRange);
            ctLineSer.addNewIdx().setVal(i);
            // 横坐标区
            CTAxDataSource cttAxDataSource = ctLineSer.addNewCat();
            ctStrRef = cttAxDataSource.addNewStrRef();
            // 选第0列,第1-6行作为横坐标区域
            String axisDataRange = new CellRangeAddress(1, dataList.size(), 0, 0).formatAsString(sheetName, true);
            ctStrRef.setF(axisDataRange);
            // 数据区域
            CTNumDataSource ctNumDataSource = ctLineSer.addNewVal();
            CTNumRef ctNumRef = ctNumDataSource.addNewNumRef();
            // 选第1-6行,第1-3列作为数据区域 //1 2 3
            String numDataRange = new CellRangeAddress(1, dataList.size(), i + 1, i + 1).formatAsString(sheetName,
                    true);
            System.out.println(numDataRange);
            ctNumRef.setF(numDataRange);
            // 设置标签格式
            ctBoolean.setVal(false);
            CTDLbls newDLbls = ctLineSer.addNewDLbls();
            newDLbls.setShowLegendKey(ctBoolean);
            ctBoolean.setVal(false);
            newDLbls.setShowVal(ctBoolean);
            ctBoolean.setVal(false);
            newDLbls.setShowCatName(ctBoolean);
            newDLbls.setShowSerName(ctBoolean);
            newDLbls.setShowPercent(ctBoolean);
            newDLbls.setShowBubbleSize(ctBoolean);
            newDLbls.setShowLeaderLines(ctBoolean);
            // 是否是平滑曲线
            CTBoolean addNewSmooth = ctLineSer.addNewSmooth();
            addNewSmooth.setVal(false);
            // 是否是堆积曲线
            CTMarker addNewMarker = ctLineSer.addNewMarker();
            CTMarkerStyle addNewSymbol = addNewMarker.addNewSymbol();
            addNewSymbol.setVal(STMarkerStyle.NONE);
        }
        // telling the BarChart that it has axes and giving them Ids
        ctLineChart.addNewAxId().setVal(123456);
        ctLineChart.addNewAxId().setVal(123457);
        // cat axis
        CTCatAx ctCatAx = ctPlotArea.addNewCatAx();
        ctCatAx.addNewAxId().setVal(123456); // id of the cat axis
        CTScaling ctScaling = ctCatAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        ctCatAx.addNewAxPos().setVal(STAxPos.B);
        ctCatAx.addNewCrossAx().setVal(123457); // id of the val axis
        ctCatAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // val axis
        CTValAx ctValAx = ctPlotArea.addNewValAx();
        ctValAx.addNewAxId().setVal(123457); // id of the val axis
        ctScaling = ctValAx.addNewScaling();
        ctScaling.addNewOrientation().setVal(STOrientation.MIN_MAX);
        ctValAx.addNewAxPos().setVal(STAxPos.L);
        ctValAx.addNewCrossAx().setVal(123456); // id of the cat axis
        ctValAx.addNewTickLblPos().setVal(STTickLblPos.NEXT_TO);
        // 是否删除主左边轴
        ctValAx.addNewDelete().setVal(false);
        // 是否删除横坐标
        if ("line-bar".equals(type)) {
            ctCatAx.addNewDelete().setVal(true);
        }
        CTLegend ctLegend = ctChart.addNewLegend();
        ctLegend.addNewLegendPos().setVal(STLegendPos.B);
        ctLegend.addNewOverlay().setVal(false);
    }

    /**
     * 写入标题
     *
     * @param sheet
     * @param titleName
     * @param dataList
     */
    private static void drawTitle(SXSSFSheet sheet, String titleName, List<Map<String, Object>> dataList, String ip) {
        //写入标题
        // 初始化表格样式
        List<CellStyle> styleList = tableStyle();
        //x写入标题
        //时间范围
        String start = String.valueOf(dataList.get(0).get("value1"));
        String end = String.valueOf(dataList.get(dataList.size() - 1).get("value1"));
        SXSSFRow row = sheet.createRow(0);
        if (null != ip) {
            row.createCell(0).setCellValue("云主机名称: " + titleName + "\r\n" + "IP地址：" + ip + "\r\n" + "时间范围: " + start + "----" + end);
        } else {
            row.createCell(0).setCellValue("名称: " + titleName + "\r\n" + "时间范围: " + start + "----" + end);
        }
        sheet.getRow(0).getCell(0).setCellStyle(styleList.get(0));
        sheet.addMergedRegion(new CellRangeAddress(0, 3, 0, 8));
    }

    /**
     * 生成图表所需数据
     *
     * @param fldNameArr
     * @param dataList
     * @return
     */
    private static SXSSFSheet drawHiddenSheetTable(List<String> fldNameArr,
                                                   List<Map<String, Object>> dataList, int num) {
        SXSSFSheet hiddenSheet = workbook.createSheet("资源数值隐藏" + num + "页");
        // 测试时返回值
        // 初始化表格样式
        List<CellStyle> styleList = tableStyle();
        // 根据数据创建excel第一行标题行
        SXSSFRow row0 = hiddenSheet.createRow(0);
        row0.createCell(0).setCellValue("时间点");
        row0.getCell(0).setCellStyle(styleList.get(0));
        for (int i = 1; i < fldNameArr.size() + 1; i++) {
            // 设置标题
            row0.createCell(i).setCellValue(fldNameArr.get(i - 1));
            // 设置标题行样式
            row0.getCell(i).setCellStyle(styleList.get(0));
        }
        // 填充数据
        for (int i = 0; i < dataList.size(); i++) {
            // 获取每一项的数据
            Map<String, Object> data = dataList.get(i);
            // 设置每一行的字段标题和数据
            SXSSFRow rowi = hiddenSheet.createRow(i + 1);
            for (int j = 0; j < data.size(); j++) {
                // 判断是否是标题字段列
                if (j == 0) {
                    rowi.createCell(j).setCellValue((String) data.get("value" + (j + 1)));
                    // 设置左边字段样式
                    hiddenSheet.getRow(i + 1).getCell(j).setCellStyle(styleList.get(0));
                } else {
                    rowi.createCell(j).setCellValue((Double) data.get("value" + (j + 1)));
                    // 设置数据样式
                    hiddenSheet.getRow(i + 1).getCell(j).setCellStyle(styleList.get(2));
                }
            }
        }
        return hiddenSheet;
    }

    /**
     * 生成样式
     *
     * @return
     */
    private static List<CellStyle> tableStyle() {
        List<CellStyle> cellStyleList = new ArrayList<CellStyle>();
        // 样式准备
        // 标题样式
        Font font = workbook.createFont();
        font.setColor(HSSFColor.HSSFColorPredefined.WHITE.getIndex());
        CellStyle style = workbook.createCellStyle();
        style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
        style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
        style.setBorderBottom(BorderStyle.THIN); // 下边框
        style.setBorderLeft(BorderStyle.THIN);// 左边框
        style.setBorderTop(BorderStyle.THIN);// 上边框
        style.setBorderRight(BorderStyle.THIN);// 右边框
        style.setAlignment(HorizontalAlignment.LEFT);
        style.setWrapText(true);
        style.setFont(font);
        cellStyleList.add(style);
        CellStyle style1 = workbook.createCellStyle();
        style1.setBorderBottom(BorderStyle.THIN); // 下边框
        style1.setBorderLeft(BorderStyle.THIN);// 左边框
        style1.setBorderTop(BorderStyle.THIN);// 上边框
        style1.setBorderRight(BorderStyle.THIN);// 右边框
        style1.setAlignment(HorizontalAlignment.CENTER);
        cellStyleList.add(style1);
        CellStyle cellStyle = workbook.createCellStyle();
        cellStyle.setBorderTop(BorderStyle.THIN);// 上边框
        cellStyle.setBorderBottom(BorderStyle.THIN); // 下边框
        cellStyle.setBorderLeft(BorderStyle.THIN);// 左边框
        cellStyle.setBorderRight(BorderStyle.THIN);// 右边框
        cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平对齐方式
        // cellStyle.setVerticalAlignment(VerticalAlignment.TOP);//垂直对齐方式
        cellStyleList.add(cellStyle);
        return cellStyleList;
    }

    /**
     * @param jsObjects1 扇形图数据
     * @param jsObjects2 折线图数据
     * @return org.apache.poi.xssf.streaming.SXSSFWorkbook
     * @Author: lxy
     * @Description: 账单分析图标生成
     * @Date 2021/6/1 19:32
     * @Param type
     * @throws:
     **/
    public static SXSSFWorkbook createAnalyzeExcel(String titleName, List<JSONObject> jsObjects1, JSONObject jsObjects2) {
        try {
            workbook = new SXSSFWorkbook();
            List<String> key1 = new ArrayList<>();
            List<Double> value1 = new ArrayList<>();
            jsObjects1.stream().forEach(
                    a -> {
                        //过滤名字为空并且值为0的数据
                        if (null != a.get("name") && Double.valueOf(a.get("value").toString()) != 0) {
                            key1.add(a.get("name").toString());
                            value1.add(Double.valueOf(a.get("value").toString()));
                        }
                    }
            );
            creatPipChart(titleName, key1, value1, true);
            //处理图表数据
            //图标对应数值
            List<JSONObject> values = jsObjects2.getJSONArray("values").toJavaList(JSONObject.class);
            //图标横坐标--时间
            List<String> keys = jsObjects2.getJSONArray("keys").toJavaList(String.class);
            List<String> fldNameArr = Lists.newArrayList();
            List<Map<String, Object>> dataList = Lists.newArrayList();
            //取所有的类别
            for (JSONObject value : values) {
                fldNameArr.add(String.valueOf(value.getString("name")));
            }
            int index = 0;
            for (String key : keys) {
                Map<String, Object> dataMap = Maps.newHashMap();
                dataMap.put("value1", key);
                int num = 2;
                for (JSONObject value : values) {
                    List<Double> time = value.getJSONArray("data").toJavaList(Double.class);
                    dataMap.put("value" + num, time.get(index));
                    num++;
                }
                dataList.add(dataMap);
                index++;
            }
            //创建图表引用数据
            SXSSFSheet hiddenSheet = drawHiddenSheetTable(fldNameArr, dataList, 1);
            //创建折线图
            createTimeXYChar(hiddenSheet, fldNameArr, dataList, titleName, null);
            //创建柱状图
            createBarChart(hiddenSheet, fldNameArr, dataList, titleName, null, false);
            workbook.setSheetHidden(1, true);
            return workbook;
        } catch (Exception e) {
            log.error("创建excel失败{}", e);
            return workbook;
        }
    }

    private static void creatPipChart(String titleName, List<String> key1, List<Double> value1, Boolean isDlbls) {
        //创建表数据展示页
        SXSSFSheet sheet = workbook.createSheet("扇形图图表页");
        drawPipSheetMap(sheet, key1, value1, titleName, 1, isDlbls);

    }

    /**
     * @param key1      扇形图数据key
     * @param value     扇形图数据value
     * @param titleName 图标名称
     * @param num
     * @param isDlbls   是否开启数据标签
     * @return void
     * @Author: lxy
     * @Description:
     * @Date 2021/6/1 20:48
     * @Param sheet
     * @throws:
     **/
    private static void drawPipSheetMap(SXSSFSheet sheet, List<String> key1,
                                        List<Double> value, String titleName, int num, Boolean isDlbls) {
        // 创建一个画布
        Drawing<?> drawing = sheet.createDrawingPatriarch();
        // 画一个图区域
        // 前四个默认0，从第8行到第25行,从第0列到第6列的区域1
        ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 0, 4 + 30 * (num - 1), 15, 30 * num);
        // 创建一个chart对象

        XSSFChart chart = ((XSSFDrawing) drawing).createChart(anchor);
        chart.setTitleText(titleName);
        // 获取CTChart对象
        CTChart ctChart = chart.getCTChart();
        CTPlotArea ctPlotArea = ctChart.getPlotArea();
        //图例位置
        ChartLegend orCreateLegend = chart.getOrCreateLegend();
        orCreateLegend.setPosition(LegendPosition.RIGHT);

        // 饼状图
        CTPieChart ctPieChart = ctPlotArea.addNewPieChart();
        ctPieChart.addNewVaryColors().setVal(true);
        CTPieSer cTPieSer = ctPieChart.addNewSer();
        cTPieSer.addNewIdx().setVal(0);
        //图例区
        CTStrRef cTStrRef = cTPieSer.addNewTx().addNewStrRef();
        String legendDataRange = new CellRangeAddress(0, 0, num + 1, num + 1).toString();
        cTStrRef.setF(legendDataRange);
        cTStrRef.addNewStrCache().addNewPtCount().setVal(1);
        CTStrVal cTStrVal = cTStrRef.getStrCache().addNewPt();
        cTStrVal.setIdx(0);
        cTStrVal.setV("Val");

        cTStrRef = cTPieSer.addNewCat().addNewStrRef();
        String axisDataRange = new CellRangeAddress(1, key1.size(), 0, 0).toString();
        cTStrRef.setF(axisDataRange);

        cTStrRef.addNewStrCache().addNewPtCount().setVal(key1.size());
        for (int r = 0; r < key1.size(); r++) {
            cTStrVal = cTStrRef.getStrCache().addNewPt();
            cTStrVal.setIdx(r);
            cTStrVal.setV(key1.get(r));
        }

        CTNumRef cTNumRef = cTPieSer.addNewVal().addNewNumRef();
        String numDataRange = new CellRangeAddress(1, key1.size(), num + 1, num + 1).toString();
        cTNumRef.setF(numDataRange);
        cTNumRef.addNewNumCache().addNewPtCount().setVal(value.size());
        for (int r = 0; r < value.size(); r++) {
            CTNumVal cTNumVal = cTNumRef.getNumCache().addNewPt();
            cTNumVal.setIdx(r);
            cTNumVal.setV(String.valueOf(value.get(r)));
        }
        //是否开启数据标签
        if (isDlbls) {
            //添加数据标签格式，默认选项全部选中
            CTDLbls ctdlbls = ctPieChart.addNewDLbls();
            //设置数据标签格式为不选中
            CTBoolean ctBooleanForShowSerName = ctdlbls.addNewShowSerName();
            ctBooleanForShowSerName.setVal(false);
        }
    }

}
